#!/usr/bin/env python
#
# 
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/Utilities24/compute_rms_between_methods.py,v 1.6 2012/04/05 22:24:27 rhuey Exp $
#
import os, glob
import numpy.oldnumeric as Numeric
from math import sqrt

from MolKit import Read
from AutoDockTools.Docking import Docking
from mglutil.math.rmsd import RMSDCalculator


def dist(coords1, coords2):
    """return distance between two atoms, a and b.
    """
    npts = len(coords1)
    pt1 = Numeric.add.reduce(coords1)/npts
    pt2 = Numeric.add.reduce(coords2)/npts
    d = Numeric.array(pt1) - Numeric.array(pt2)
    return sqrt(Numeric.sum(d*d))


if __name__ == '__main__':
    import sys
    import getopt


    def usage():
        print "Usage: compute_rms_between_methods.py -f reference directory -d directory"
        print "  Compare two docking methods by computing rms of docked conformations:LE and LE_LC"
        print "   By default, output written to 'summary_rms_results'"
        print
        print "    Description of command..."
        print "         -f     reference directory"
        print "         -d     directory"
        print "    Optional parameters:"
        print "        [-t]    rmsd tolerance (default is 1.5)"
        print "        [-o]    output filename"
        print "                (default is 'summary_rms_results')"
        print "        [-v]    verbose output"


    # process command arguments
    try:
        opt_list, args = getopt.getopt(sys.argv[1:], 'f:d:o:t:vh')
    except getopt.GetoptError, msg:
        print 'compute_rms_between_methods.py: %s' %msg
        usage()
        sys.exit(2)

    # initialize required parameters
    #-f: refdirectory
    refdirectory =  None
    #-d: directory
    directory =  None
    #-t: rms_tolerance
    rms_tolerance =  1.5
    # optional parameters
    verbose = None
    #-o outputfilename
    outputfilename = "summary_rms_results"

    #'f:d:o:t:vh'
    for o, a in opt_list:
        #print "o=", o, " a=", a
        if o in ('-f', '--f'):
            refdirectory = a
            if verbose: print 'set refdirectory to ', a
        if o in ('-d', '--d'):
            directory = a
            if verbose: print 'set directory to ', a
        if o in ('-o', '--o'):
            outputfilename = a
            if verbose: print 'set outputfilename to ', a
        if o in ('-t', '--t'):
            rms_tolerance = float(a)
            if verbose: print 'set rms_tolerance to ', a
        if o in ('-v', '--v'):
            verbose = True
            if verbose: print 'set verbose to ', True
        if o in ('-h', '--'):
            usage()
            sys.exit()

    msg = ""
    if not refdirectory:
        msg += 'reference directory must be specified!\n'
        if not directory: 
            msg = 'Both reference directory and directory must be specified!\n'
    elif not directory:
        msg += 'directory must be specified!'

    if not refdirectory or not directory:
        print msg
        usage()
        sys.exit()

    #process docking in reference directory
    #read all the docking logs in reference directory as one Docking
    ref_dlg_list = glob.glob(refdirectory + '/*.dlg')
    ref_d = Docking()
    for dlg in ref_dlg_list:
        ref_d.readDlg(dlg)
    #setup rmsd tool
    coords = ref_d.ligMol.allAtoms.coords[:]
    ref_d.clusterer.rmsTool = RMSDCalculator(coords)
    ref_d.clusterer.make_clustering(rms_tolerance) 
    clust0 = ref_d.clusterer.clustering_dict[rms_tolerance][0]
    c = clust0[0] #lowest energy overall in reference docking
    ref_d.ch.set_conformation(c)
    ref_LE_coords = ref_d.ligMol.allAtoms.coords[:]
    ref_largest = clust0
    for clust in ref_d.clusterer.clustering_dict[rms_tolerance]:
        if verbose: print "current largest cluster len= ", len(clust)
        if len(clust)>len(ref_largest): 
            if verbose: print "resetting largest clust: now len=", len(clust)
            ref_largest = clust
    ref_d.ch.set_conformation(ref_largest[0])
    ref_LC_coords = ref_d.ligMol.allAtoms.coords[:]


    #process docking in test directory
    dlg_list = glob.glob(directory + '/*.dlg')
    d = Docking()
    for dlg in dlg_list:
        d.readDlg(dlg)
    #setup rmsd tool
    #coords = d.ligMol.allAtoms.coords[:]
    d.clusterer.rmsTool = RMSDCalculator(coords)
    d.clusterer.make_clustering(rms_tolerance) 
    clust0 = d.clusterer.clustering_dict[rms_tolerance][0]
    c = clust0[0]
    #SET d.ligMol to LOWEST ENERGY conf
    d.ch.set_conformation(c)
    LE_coords = d.ligMol.allAtoms.coords[:]
    # lowest energy in test v. lowest energy in ref
    rms_LE_LE = c.getRMSD(ref_LE_coords)
    dist_LE_LE = dist(ref_LE_coords, LE_coords) 
    #repeat for ref LC coords
    # lowest energy in test v. largest cluster lowest energy in ref
    rms_LE_LC = c.getRMSD(ref_LC_coords)
    dist_LE_LC = dist(ref_LC_coords, LE_coords) 

    # repeat for largest cluster 
    largest = clust0
    for clust in d.clusterer.clustering_dict[rms_tolerance]:
        if verbose: print "current largest cluster len= ", len(clust)
        if len(clust)>len(largest): 
            if verbose: print "resetting largest clust: now len=", len(clust)
            largest = clust
    conf_LC = largest[0]  #lowest energy conformation in largest cluster
    #SET d.ligMol to LARGEST CLUSTER's LOWEST ENERGY conf
    d.ch.set_conformation(conf_LC)
    LC_coords = d.ligMol.allAtoms.coords[:]
    # largest cluster lowest energy in test v. lowest energy in ref
    rms_LC_LE = conf_LC.getRMSD(ref_LE_coords)
    dist_LC_LE = dist(LC_coords, ref_LE_coords)
    #repeat for ref LC coords
    # largest cluster lowest energy in test v. largest cluster lowest energy in ref
    rms_LC_LC = conf_LC.getRMSD(ref_LC_coords)
    dist_LC_LC = dist(LC_coords, ref_LC_coords)
    

    first = not os.path.exists(outputfilename)
    if first:
        fptr = open(outputfilename, 'w')
        ostr= "# (in human terms: test_le->ref_le     test_le ->ref_lc        test_lc->ref_lc    test_lc ->ref_le)\n"
        fptr.write(ostr)
        ostr = "# ligand      rms_LE_LE dist_LE_LE  rms_LE_LC dist_LE_LC  rms_LC_LC dist_LC_LC  rms_LC_LE dist_LC_LE\n"
        fptr.write(ostr)
    else:
        fptr = open(outputfilename, 'a')
   
    ostr = "% 10s,% 10.4f,% 10.4f,% 10.4f,% 10.4f,% 10.4f,% 10.4f,% 10.4f,% 10.4f,\n" %(d.ligMol.name,\
                rms_LE_LE, dist_LE_LE, rms_LE_LC, dist_LE_LC, \
                rms_LC_LC, dist_LC_LC, rms_LC_LE, dist_LC_LE)
    fptr.write(ostr)
    fptr.close()
    

# To execute this command type:
# compute_rms_between_methods.py -f refdirectory -d directory -t rmsd tolerance 
#                         -v verbose
# NOTE: -f, -d and -t require arguments whereas the other, -v, sets a boolean




#############################################################################
#
# Author: Ruth HUEY, Michel F. SANNER
#
# Copyright: M. Sanner TSRI 2000
#
#############################################################################
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/__init__.py,v 1.54 2011/06/30 20:37:27 sargis Exp $
#
# $Id: __init__.py,v 1.54 2011/06/30 20:37:27 sargis Exp $
# this makes this directory a package

import os
import sys
import getopt
import time
import socket
from string import split

from Support.version import __version__
from mglutil import __revision__

# create hostDict with hostMacros accessible by anyone
from AutoDockTools.adthosts import hostMacros
from AutoDockTools.autodockHosts import AutoDockHosts

hostDict = AutoDockHosts(hostMacros)

h= socket.gethostname()
hostDict[h]=hostDict['localhost']
hostDict[h]['host']=h
del hostDict['localhost']

# try to extend that dictionary with user specific hostMacros

# first try to find a adthost file in current directory
if os.path.isfile('./adthosts.py'):
    execfile('./adthosts.py')
    if globals().has_key('hostMacros'):
        hostDict.update(hostMacros)
elif os.name != 'nt': #sys.platform!='win32':
    # try to find the user's home directory
    import posix
    if 'HOME' in posix.environ.keys():
        try:
            execfile(os.path.join(posix.environ['HOME'],'adthosts.py'))
            if globals().has_key('hostMacros'):
                hostDict.update(hostMacros)
        except:
            pass
   

def setADTmode(modeStr, mv):
    import Tkinter
    from AutoDockTools.autotorsCommands import AdtSetMode
    mv.addCommand(AdtSetMode(),'ADTSetMode')
    if not hasattr(mv.GUI, 'adtBar'):
        mv.GUI.adtBar = mv.GUI.menuBars['AutoToolsBar']
    if not hasattr(mv.GUI, 'adtFrame'):
        mv.GUI.adtFrame = vf.GUI.adtBar.menubuttons.values()[0].master
    if not hasattr(mv.GUI, 'adt41ModeLabel'):
        mv.GUI.adt41ModeLabel=Tkinter.Label(mv.GUI.adtFrame, text="ADT4.2", width=6,
                             relief='sunken', borderwidth=1, fg='DarkGreen',
                             bg = 'ivory',anchor='w' )
        mv.GUI.adt41ModeLabel.pack(side='left')
    mv.GUI.adt41ModeLabel.bind("<Double-Button-1>", mv.ADTSetMode.guiCallback)
    mv.ADTSetMode.Close_cb()
    return mv.ADTSetMode(modeStr)
     

def setdmode(mode, mv):
    """
load display commands for mode and set them as default command for new molecule
"""
    if mode=='cpk':
        mv.browseCommands('displayCommands', commands=['displayCPK'],
              log=0, package='Pmv')
        mv.addOnAddObjectCmd(mv.displayCPK)

    elif mode=='lines':
        mv.browseCommands('bondsCommands',
              commands=('buildBondsByDistance',), log=0)
        mv.addOnAddObjectCmd(mv.buildBondsByDistance)
        mv.addOnAddObjectCmd(mv.displayLines)

    elif mode=='ss':
        mv.browseCommands('secondaryStructureCommands',
              commands=('ribbon',), log=0)
        mv.addOnAddObjectCmd(mv.ribbon)

    elif mode=='sb':
        mv.browseCommands('bondsCommands',
                          commands=('buildBondsByDistance',), log=0)
        mv.addOnAddObjectCmd(mv.buildBondsByDistance)
        mv.browseCommands('displayCommands',
                          commands=('displaySticksAndBalls',), log=0)
        mv.addOnAddObjectCmd(mv.displaySticksAndBalls, (),
                             {'sticksBallsLicorice':'SticksAndBalls', 'cquality':0, 'bquality':0})

    elif mode=='lic':
        mv.browseCommands('bondsCommands',
                          commands=('buildBondsByDistance',), log=0)
        mv.addOnAddObjectCmd(mv.buildBondsByDistance)
        mv.browseCommands('displayCommands',
                          commands=('displaySticksAndBalls',), log=0)
        mv.addOnAddObjectCmd(mv.displaySticksAndBalls, (),
                             {'sticksBallsLicorice':'Licorice', 'cquality':0, 'cradius':.2})

    elif mode=='ms':
        mv.browseCommands('msmsCommands', commands=('computeMSMS',), log=0)
        mv.browseCommands('msmsCommands', commands=('displayMSMS',), log=0)
        mv.addOnAddObjectCmd(mv.computeMSMS, (), {'density':3.0})
        mv.addOnAddObjectCmd(mv.displayMSMS)
    
    elif mode=='ca':
        mv.browseCommands('traceCommands', commands=('computeTrace',), log=0)
        mv.browseCommands('traceCommands', commands=('extrudeTrace',), log=0)
        mv.browseCommands('traceCommands', commands=('displayTrace',), log=0)
        mv.addOnAddObjectCmd(mv.computeTrace)
        mv.addOnAddObjectCmd(mv.extrudeTrace)
        mv.addOnAddObjectCmd(mv.displayTrace)

    elif mode=='bt':
        mv.browseCommands('bondsCommands',
              commands=('buildBondsByDistance',), log=0)
        mv.browseCommands('displayCommands',
              commands=('displayBackboneTrace',), log=0)
        mv.addOnAddObjectCmd(mv.displayBackboneTrace, (), 
                 {'cquality':0, 'bquality':0, 'cradius':0.25,
                  'bRad':0.33} )
        
    elif mode=='sp':
        mv.browseCommands('splineCommands', commands=('computeSpline',), log=0)
        mv.browseCommands('splineCommands', commands=('extrudeSpline',), log=0)
        mv.browseCommands('splineCommands',
              commands=('displayExtrudedSpline',), log=0)
        mv.addOnAddObjectCmd(mv.computeSpline)
        mv.addOnAddObjectCmd(mv.extrudeSpline)
        mv.addOnAddObjectCmd(mv.displayExtrudedSpline)

    elif mode=='sssb':
        mv.browseCommands('displayCommands',
              commands=('displaySSSB',), log=0, package='Pmv')
        mv.addOnAddObjectCmd(mv.displaySSSB)


def setcmode(mode, mv):
    """
load color commands for mode and set them as default command for new molecule
"""
    if mode=='ca':
        mv.browseCommands('colorCommands', commands=('colorByAtomType',),
              log=0, package='Pmv')
        mv.addOnAddObjectCmd(mv.colorByAtomType)

    elif mode=='cr':
        mv.browseCommands('colorCommands',
              commands=('colorByResidueType',), log=0)
        mv.addOnAddObjectCmd(mv.colorByResidueType)
        
    elif mode=='cc':
        mv.browseCommands('colorCommands', commands=('colorByChains',),
              log=0, package='Pmv')
        mv.addOnAddObjectCmd(mv.colorByChains)

    elif mode=='cm':
        mv.browseCommands('colorCommands', commands=('colorByMolecules',),
                   log=0, package='Pmv')
        mv.addOnAddObjectCmd(mv.colorByMolecules)

    elif mode=='cdg':
        mv.browseCommands('colorCommands', commands=('colorAtomsUsingDG',),
              log=0, package='Pmv')
        mv.addOnAddObjectCmd(mv.colorAtomsUsingDG)

    elif mode=='cs':
        mv.browseCommands('colorCommands',
              commands=('colorResiduesUsingShapely',), log=0 )
        mv.addOnAddObjectCmd(mv.colorResiduesUsingShapely)

    elif mode=='css':
        mv.browseCommands('secondaryStructureCommands',
              commands=('colorBySecondaryStructure',), log=0)
        mv.addOnAddObjectCmd(mv.colorBySecondaryStructure)


##################################################################
# Define a bunch of useful methods
##################################################################
def ed():
    return mv.vision.ed


def net():
    return mv.vision.ed.currentNetwork


def nodes():
    return mv.vision.ed.currentNetwork.nodes


def runADT(*argv, **kw):
    """The main function for running AutoDockTools
"""
    import sys

    if type(argv) is tuple:
        if len(argv) == 0:
            argv = None
        elif len(argv) == 1:
            argv = argv[0]
            if type(argv) is not list:
                argv = [argv]
        else:
            argv = list(argv)
    if kw.has_key("ownInterpreter"):
        ownInterpreter = kw["ownInterpreter"]
    else:
        if argv is None:
            argv = ['AutoDockTools/bin/runADT.py', '-i']
            ownInterpreter = False
        elif argv[0].endswith('runADT.py') is False:
            argv.insert(0,'-i')
            argv.insert(0,'AutoDockTools/bin/runADT.py')
            ownInterpreter = False
        else:
            ownInterpreter = True

    optlist, args = getopt.getopt(argv[1:], 'haipsd:c:v:', [
        'update', 'help', 'again', 'overwriteLog', 'uniqueLog', 'noLog',
        'noGUI', 'die', 'customizer=', 'interactive', 'dmode=', 'cmode=',
        'noSplash', 'vision', 'python'] )
    
    help_msg = """usage: pmv <options>
            -h or --help          : print this message
            -a or --again         : play back lastlog file
            --overwriteLog        : overwrite log file
            --uniqueLog           : create a log file with a unique name
            --noLog               : turn off logging
            --noGUI               : start PMV without the Graphical User Interface
            -s or --noSplash      : turn off Splash Screen
            --die                 : do not start GUI event loop
            --customizer file     : run the user specified file
            --lib packageName     : add a libraries of commands
            -p or --ipython       : create an ipython shell instead of a python shell        
            -v r or --vision run  : run vision networks on the command line
            -v o or --vision once : run vision networks and exit PMV

        --update [nightly|tested|clear] : update MGLTools
                if no arguments are given Update Manager GUI is provided
                'nightly': download and install Nightly Builds
                'tested' : download and install tested Nightly Builds
                'clear'  : clear/uninstall all the updates

        -d or --dmode modes : specify a display mode
                modes can be any a combination of display mode
               'cpk'  : cpk
               'lines': lines
               'ss'   : secondary structure ribbon
               'sb'   : sticks and balls
               'lic'  : licorice
               'ms'   : molecular surface
               'ca'   : C-alpha trace
               'bt'   : backbone trace
               'sp'   : CA-spline
               'sssb' : secondary structure for proteins,
                        sticks and balls for other residues with bonds
                        lines for other residues without bonds
    
        -c or --cmode modes : specify a display mode color scheme:
                'ca' : color by atom
                'cr' : color by residue (RASMOL scheme)
                'cc' : color by chain
                'cm' : color by molecule
                'cdg': color using David Goodsell's scheme
                'cs' : color residues using Shapely scheme
                'css': color by secondary structure element

              example:
              display protein as ribbon, non protein as sticks and balls
              and color by atom type
                 adt -i --dmode sssb --cmode cr myprot.pdb
                 adt -i -m sssb -c cr myprot.pdb
    
    """

    customizer = None
    logmode = 'overwrite'
    libraries = []
    again = 0
    interactive = 0
    ipython = False
    die=0
    gui = True
    noSplash = False
    dmode = cmode = None
    dmodes = ['cpk', 'lines', 'ss', 'sb', 'lic', 'ms', 'ca', 'bt', 'sp', 'sssb' ]
    cmodes = ['ca', 'cr', 'cc', 'cm', 'cdg', 'cs', 'css']
    visionarg = None

    for opt in optlist:
        if opt[ 0] in ('-h', '--help'):
            print help_msg
            sys.exit()
        elif opt[ 0] in ('-a', '--again'):
            again = 1
            os.system("mv mvAll.log.py .tmp.py")
        elif opt[ 0] =='--overwriteLog': logmode = 'overwrite'
        elif opt[ 0] =='--uniqueLog': logmode = 'unique'
        elif opt[ 0] =='--noLog': logmode = 'no'
        elif opt[ 0] =='--noGUI': gui = False
        elif opt[ 0] =='--die': die = 1
        elif opt[ 0] in ('-s', '--noSplash'):
            noSplash = True
        elif opt[ 0] == '--customizer':
            customFile = opt[1]
        elif opt[ 0] == '--lib':
            libraries.append(opt[1])
        elif opt[ 0] in ('-i', '--interactive'):
            interactive = 1
        elif opt[ 0] in ('-p', '--python'):
            ipython = True
        elif opt[ 0] in ('-d', '--dmode'):
            assert min([mo in dmodes for mo in opt[1].split('|')])==True
            dmode = opt[1]
        elif opt[ 0] in ('-c', '--cmode'):
            assert min([mo in cmodes for mo in opt[1].split('|')])==True
            cmode = opt[1]
        elif opt[0] == '--update':
            try:
                from Support.update import Update
            except ImportError:
                print "Support package is needed to get updates"
                break
                
            update = Update()
            if 'nightly' in args:
                update.latest = 'nightly'
                update.getUpdates()
            elif 'tested' in args:
                update.latest     = 'tested'
                update.getUpdates()
            elif 'clear' in args:
                print "Removing all updates"
                update.clearUpdates()
            else:
                waitTk = update.gui()
                update.master.wait_variable(waitTk)
        elif opt[ 0] in ('-v', '--vision'):
            if opt[1] in ('o', 'once'):
                visionarg = 'once'
            elif opt[1] in ('r', 'run'):
                visionarg = 'run'
        else:
            print "unknown option %s %s"%tuple(opt)
            print help_msg
            sys.exit(1)
    
    #import sys
    text = 'Python executable     : '+ sys.executable +'\n'
    if kw.has_key('AdtScriptPath'):
        text += 'ADT script                : '+ kw['AdtScriptPath'] +'\n'
    text += 'MGLTool packages '+'\n'
    
    from Support.path import path_text, release_path
    from Support.version import __version__
    from mglutil import __revision__

    version = __version__
    text += path_text
    text += version+': '+release_path
    
    path_data = text

    print 'Run ADT from ', __path__[0]
    # if MGLPYTHONPATH environment variable exists - insert the specified path
    # into sys.path
    
    #if os.environ.has_key("MGLPYTHONPATH"):
    #    if sys.platform == "win32":
    #        mglPath = split(os.environ["MGLPYTHONPATH"], ";")
    #    else:
    #        mglPath = split(os.environ["MGLPYTHONPATH"], ":")
    #    mglPath.reverse()
    #    for p in mglPath:
    #        sys.path.insert(0, os.path.abspath(p))

    try:
        ##################################################################
        # Splash Screen
        ##################################################################
        import Pmv
        image_dir = os.path.join(  Pmv.__path__[0],'Icons','Images')
        copyright = """(c) 1999-2011 Molecular Graphics Laboratory, The Scripps Research Institute
    ALL RIGHTS RESERVED """
        authors = """Authors: Michel F. Sanner, Ruth Huey, Sargis Dallakyan,
Chris Carrillo, Kevin Chan, Sophie Coon, Alex Gillet,
Sowjanya Karnati, William (Lindy) Lindstrom, Garrett M. Morris, Brian Norledge,
Anna Omelchenko, Daniel Stoffler, Vincenzo Tschinke, Guillaume Vareille, Yong Zhao"""
        icon = os.path.join(Pmv.__path__[0],'Icons','64x64','adt.png')
        third_party = """Fast Isocontouring, Volume Rendering -- Chandrait Bajaj, UT Austin
Adaptive Poisson Bolzman Solver (APBS) -- Nathan Baker Wash. Univ. St Louis
GL extrusion Library (GLE) -- Linas Vepstas
Secondary Structure Assignment (Stride) -- Patrick Argos EMBL
Mesh Decimation (QSlim 2.0) -- Micheal Garland,  Univeristy of Illinois
Tiled Rendering (TR 1.3) -- Brian Paul
GLF font rendering library --  Roman Podobedov
PyMedia video encoder/decoder -- http://pymedia.org"""
        title="AutoDockTools"
        #create a root and hide it
        try:
            from TkinterDnD2 import TkinterDnD
            root = TkinterDnD.Tk()
        except ImportError:
            from Tkinter import Tk
            root = Tk()
        root.withdraw()
    
        from mglutil.splashregister.splashscreen import SplashScreen
        from mglutil.splashregister.about import About
        about = About(image_dir=image_dir, third_party=third_party,
                      path_data=path_data, title=title, version=version,
                      revision=__revision__,
                      copyright=copyright, authors=authors, icon=icon)
        if gui:
            splash =  SplashScreen(about, noSplash=noSplash)

        from Pmv.moleculeViewer import MoleculeViewer

        mv = MoleculeViewer(
            logMode=logmode, customizer=customizer, master=root,
            title=title, withShell= not interactive, verbose=False, gui=gui)

        mv.browseCommands('autotors41Commands', commands = None, 
                          package = 'AutoDockTools')
        mv.browseCommands('autoflex41Commands', commands = None,
                          package = 'AutoDockTools')
        mv.browseCommands('autogpf41Commands', commands = None, 
                          package = 'AutoDockTools')
        mv.browseCommands('autodpf41Commands', commands = None, 
                          package = 'AutoDockTools')
        mv.browseCommands('autostart41Commands', commands = None, 
                          package = 'AutoDockTools')
        mv.browseCommands('autoanalyze41Commands', commands = None,
                          package = 'AutoDockTools')
        #mv.GUI.currentADTBar = 'AutoTools42Bar'
        #setADTmode("AD4.0", mv)
        setADTmode("AD4.2", mv)
        mv.browseCommands('selectionCommands', package='Pmv')
        mv.browseCommands('AutoLigandCommand', package='AutoDockTools',  topCommand=0)        
        mv.GUI.naturalSize()
        mv.customize('_adtrc')

        mv.help_about = about

        if gui:
            font = mv.GUI.ROOT.option_get('font', '*')
            mv.GUI.ROOT.option_add('*font', font)    

        try:
            import Vision
            mv.browseCommands('visionCommands', commands=('vision',), topCommand=0)
            mv.browseCommands('coarseMolSurfaceCommands', topCommand=0)
            if hasattr(mv,'vision') and mv.vision.ed is None:
                mv.vision(log=0)
            else:
                # we address the global variable in vision
                Vision.ed = mv.vision.ed
        except ImportError:
            pass

        #show the application after it built
        if gui:
            splash.finish()
            root.deiconify()
        globals().update(locals())

        if gui:
            mv.GUI.VIEWER.suspendRedraw = True
        cwd = os.getcwd() 
        #mv._cwd differs from cwd when 'Startup Directory' userpref is set
        os.chdir(mv._cwd)
        if dmode is not None or cmode is not None:
            # save current list of commands run when a molecule is loaded
            addCmds = mv.getOnAddObjectCmd()
        # remove them
            if dmode is not None:
                for c in addCmds:
                    mv.removeOnAddObjectCmd(c[0])
                # set the mode
                setdmode(dmode, mv)
    
            if cmode is not None:
            # set the mode
                setcmode(cmode, mv)
    
        for a in args:
            if a[0]=='-':# skip all command line options
                continue
    
            elif (a[-10:]=='_pmvnet.py') or (a[-7:]=='_net.py'):  # Vision networks
                mv.browseCommands('visionCommands', commands=('vision',) )
                if mv.vision.ed is None:
                    mv.vision()
                mv.vision.ed.loadNetwork(a)
                if visionarg == 'run' or visionarg == 'once':
                    mv.vision.ed.softrunCurrentNet_cb()

            elif a[-3:]=='.py':     # command script
                print 'sourcing', a
                mv.source(a)
    
            elif a[-4:] in ['.pdb', '.pqr', 'pdbq', 'mol2', '.cif', '.gro'] or a[-5:]=='pdbqs' or a[-5:]=='pdbqt':
                mv.readMolecule(a)

            elif a in ['clear', 'tested', 'nighlty']:
                pass

            else:
                print 'WARNING: unable to process %s command line argument'%a
        if again:
            mv.source(".tmp.py")
        if dmode is not None or cmode is not None:
            # get current list of commands run when a molecule is loaded
            cmds = mv.getOnAddObjectCmd()
            # remove them
            for c in cmds:
                mv.removeOnAddObjectCmd(c[0])
            # restore original list of commands
            for c in addCmds:
                apply( mv.addOnAddObjectCmd, c )

        if gui:
            mv.GUI.VIEWER.suspendRedraw = False
        os.chdir(cwd)
        if visionarg != 'once':
            if ownInterpreter is True:
                mod = __import__('__main__')
                mod.__dict__.update({'self':mv})
                if interactive:
                    sys.stdin = sys.__stdin__
                    sys.stdout = sys.__stdout__
                    sys.stderr = sys.__stderr__
                    if ipython is True:
                        try:
                            # create IPython shell
                            from IPython.Shell import _select_shell
                            sh = _select_shell([])(argv=[], user_ns=mod.__dict__)
                            sh.mainloop()
                        except:
                            import code
                            try: # hack to really exit code.interact 
                                code.interact( 'AutoDockTools Interactive Shell', local=mod.__dict__)
                            except:
                                pass
                    else:
                        import code
                        try: # hack to really exit code.interact 
                            code.interact( 'AutoDockTools Interactive Shell', local=mod.__dict__)
                        except:
                            pass
                elif not die:
                    #mv.GUI.pyshell.interp.locals = globals()
                    if gui:
                        mv.GUI.pyshell.interp.locals = mod.__dict__
                        mv.GUI.ROOT.mainloop()
                mod.__dict__.pop('self')
        else:
            ed.master.mainloop()
    except:
        import traceback
        traceback.print_exc()
        raw_input("hit enter to continue")
        import sys
        sys.exit(1)


packageContainsVFCommands = 1

CRITICAL_DEPENDENCIES =['numpy', 'Pmw', 'mglutil', 'ViewerFramework', 'MolKit', 'DejaVu', 'opengltk', 'Pmv']
NONCRITICAL_DEPENDENCIES =['ZSI', 'geomutils', 'UTpackages', 'SpatialLogic', 'bhtree', 'sff', 'PyBabel', 'Volume', 'mslib', 'Vision', 'NetworkEditor', 'ARTK', 'PIL', 'symserv','QSlimLib', 'AutoDockTools', 'PyMead', 'isocontour','Support', 'scenario', 'python_cluster', 'IPython', 'PyAutoDock']



## Automatically adapted for numpy.oldnumeric Jul 23, 2007 by 

#
# Copyright: M. Sanner TSRI 2002
#
# Last modified on Wed May 29 12:28:00 PDT 2002 by lindy
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/Conformation.py,v 1.48 2012/03/30 19:56:34 rhuey Exp $
#

import math
import string
import numpy.oldnumeric as Numeric

from mglutil.math.rmsd import RMSDCalculator
from mglutil.math.statetocoords import StateToCoords
from mglutil.math.transformation import Transformation
from AutoDockTools.cluster import Clusterer
from AutoDockTools.ResultParser import ResultParser



class Conformation:
    """This class can be sent to StateToCoord because attr's match

    """
    def __init__(self, mol, origin, translation, quaternion, torsions,
                 coords = None):
        self.mol = mol
        self.origin = origin
        self.translation = translation
        self.quaternion = quaternion
        self.torsions = torsions
        if coords:
            self.coords = coords
        # the way of the future is (I think) for there to be
        # a state attr and coords attr and a Conformation
        # could be instantiated with either.
        #   do this when stoc is integrated so that stoc.applyState
        #   and deal with it.

        # the cluster dict is keyed by tolerance
        # a conformation knows what cluster(s) it belongs to...
        self.cluster_dict = {}


    def getTorsionOnlyCoords(self):
        """Return your coordinates with no quaternion.

        Don't save these coords, compute them every time.
        """
        self.mol.allAtoms.setConformation(self.mol.stoc.confIndex)
        self.mol.stoc.applyAngList(self.torsions,
            Transformation(trans=self.origin).getMatrix(transpose=1))
        return self.mol.allAtoms.coords


    def getCoords(self):
        """Return your coordinates.

        If the coordinates haven't been computed yet,
            then compute, save, and return them.
        Otherwise, return the previously-computed coordinates.
        """
        #FIX THIS: how could it be None? or []?
        if not hasattr(self, 'coords') or self.coords is None or len(self.coords)==0:
            # then compute the coords
            oldCoords = self.mol.allAtoms.coords[:]
            oldConf = self.mol.allAtoms[0].conformation
            self.mol.allAtoms.setConformation(self.mol.stoc.confIndex)
            self.mol.stoc.applyState(self) # !!! attr's must match !!!
            # and save the coords (this makes a real copy of coords)
            self.coords = Numeric.array(self.mol.allAtoms.coords).tolist()
            self.mol.allAtoms.updateCoords(oldCoords, oldConf)
        return self.coords


    def getRMSD(self, refCoords=None, numCoords=None):
        """Return RMSD of this conformations relative to refCoords.

        If refCoords is not given, the original coordinates for the
        molecule will be used as the reference.
        """
        if not refCoords:
            oldCoords = self.mol.allAtoms.coords[:]
            oldConf = self.mol.allAtoms[0].conformation
            self.mol.allAtoms.setConformation(0)
            refCoords = self.mol.allAtoms.coords[:]
            self.mol.allAtoms.updateCoords(oldCoords, oldConf)
        rmsd_calc = RMSDCalculator(refCoords)            
        if numCoords:
            rmsd = rmsd_calc.computeRMSD(self.getCoords()[:numCoords])
        else:
            rmsd = rmsd_calc.computeRMSD(self.getCoords())
        return rmsd


    def getRMSD_custom(self, refCoords=None):
        """Return the minimum of the regular RMSD and the
        computed RMSD after the coords have been rotated about
        the c2 axis which was aligned with the y-axis.
        """
        normal_RMSD = self.getRMSD(refCoords)

        c2_coords = self.coords[:]
        # I know there's a clever way to use Numeric to do this...
        for c in c2_coords:
            c[0] = -1.0 * c[0]
            c[2] = -1.0 * c[2]

        if not refCoords:
            self.mol.allAtoms.setConformation(0)
            refCoords = self.mol.allAtoms.coords
        rmsd_calc = RMSDCalculator(refCoords)            
        c2_RMSD = rmsd_calc.computeRMSD(self.getCoords())
        return min( normal_RMSD, c2_RMSD)


    def getCoords_subset(self):
        """Return coordinates of current subset, if there is one.

        If the coordinates haven't been computed yet,
            then compute, save, and return them.
        Otherwise, return the previously-computed coordinates.
        """
        if not self.subset:
            return self.getCoords()
        if not hasattr(self, 'subset_coords'):
            oldCoords = self.mol.allAtoms.coords[:]
            oldConf = self.mol.allAtoms[0].conformation
            #by default use the last coords 
            index_to_use = len(self.mol.allAtoms[0]._coords) - 1  
            if hasattr(self.mol, 'stoc'):
                index_to_use = self.mol.stoc.confIndex
            self.mol.allAtoms.updateCoords(self.getCoords(), 
                        index_to_use) # previously: self.mol.stoc.confIndex)
            self.subset_coords = self.subset.coords[:]
            self.mol.allAtoms.updateCoords(oldCoords, oldConf)
        return self.subset_coords


    def getRMSD_subset(self, refCoords=None):
        """Return RMSD of this conformations subset relative to refCoords.

        If refCoords is not given, the original coordinates for the
        subset will be used as the reference.
        """
        if not refCoords:
            refCoords = self.getCoords_subset()
        rmsd_calc = RMSDCalculator(refCoords)            
        return rmsd_calc.computeRMSD(self.getCoords_subset())


    def writeTrj(self, fileptr, istep=1, lastmove='a'):
        """Supply a file_handle and this conformation instance
        will write itself out.
        """
        #FIX THIS: which energy should be written here???
        binding_energy = self.binding_energy
        docking_energy = self.docking_energy
        if not binding_energy:
            binding_energy = 0.000
        if not docking_energy:
            docking_energy = 0.000
        #have to add back the origin to return to AutoDock space
        fileptr.write("state %d %c %f %f %f %f %f %f %f %f %f\n" %
                   (istep,
                    lastmove,
                    binding_energy,
                    docking_energy,
                    self.translation[0] + self.origin[0],
                    self.translation[1] + self.origin[1],
                    self.translation[2] + self.origin[2],
                    self.quaternion[0],
                    self.quaternion[1],
                    self.quaternion[2],
                    self.quaternion[3]))
        for tor in self.torsions:
            fileptr.write("%f\n" % tor)


    def writeRes101(self, fileptr): 
        #data_run_id:
        #1784593 
        #dpf_id:
        #1 
        #ei_version:
        #1.01 
        #ag_version ad_version:
        #3.00 3.05 
        #run_rank: rank of this run in cluster
        #4 
        #run_number: number of this run in cluster
        #6 
        #cluster_rank: rank of this cluster
        #4
        #cluster_size: number of conformations in this cluster
        #1 
        #run_size: number of runs specified in dpf
        #10 
        #rseed1 and rseed2
        #965794405 1006315922 
        #rmsd: RMSD from reference structure
        #26.255011 
        #binding_energy: estimated free energy of binding
        #-8.846132 
        #docking_energy: final docked energy
        #-9.836534 
        #translation x, y, z
        #2.805229 1.033521 1.957878 
        #quaternion unit vector x, y, z, w
        #0.563571 -0.554028 0.612732 -135.133343 
        #number of torsions
        #13 
        #torsion values:
        #-140.26 171.40 -48.43 150.90 -10.93 105.64 -155.73 148.22 174.48 -116.86 -105.52 139.56 -110.30 
        fileptr.write("17 1 1.01 3.00 3.05")
        #nb: conformation.run is the same as run_number
        #nb2:have to add back the origin to return to AutoDock space
        for item in ['run_rank', 'run', 'cluster_rank', 'cluster_size',
                    'run_size', 'rseed1', 'rseed2']:
            if not hasattr(self, item):
                fileptr.write(' 1')
            elif getattr(self, item):
                fileptr.write(" %d" %getattr(self, item))
            else:
                fileptr.write(" 10")
        #write the RMSD for this conformation
        fileptr.write(" %f" %(round(self.getRMSD(),6)))
        for item in [self.binding_energy, self.docking_energy, 
                    self.translation[0] + self.origin[0], 
                    self.translation[1] + self.origin[1], 
                    self.translation[2] + self.origin[2], 
                    self.quaternion[0], self.quaternion[1], 
                    self.quaternion[2], self.quaternion[3]]:
            fileptr.write(" %f" %item)
        fileptr.write(" %d" %len(self.torsions))
        for tor in self.torsions:
            fileptr.write(" %f" % tor)
        #at the end add a newline
        fileptr.write('\n')

    

    def writeRes(self, fileptr):
        #eg
        #9033311 116126 10 1/30/2001 7:27:34 AM 1/30/2001 7:27:34 AM 1.00 3.00 3.05 3 18 3 1 20 980868135 18826794 4.979233 -16.653694 -18.776157 5.005155 -2.431130 15.439371 0.009171 0.617751 0.786320 -170.668379 12 -67.91 -107.72 101.99 -56.92 176.25 21.67 48.10 24.66 -3.65 12.54 -10.66 -3.68 
        #this is for version 1.00
        #1: output_id: %d
        #9033311 
        #2: data_run_id: %d
        #116126 
        #3: dpf_id: %d
        #10 
        #4: creation_dtime: 3part
        #1/30/2001 7:27:34 AM 
        #5: last_update_dtime: 3part
        #1/30/2001 7:27:34 AM 
        #6: ei_version: %f
        #1.00 
        #7: ag_version: %f
        #3.00 
        #8: ad_version: %f
        #3.05 
        #9: run_rank:  %d
        #3 
        #10: run_number:    %d
        #18 
        #11: cluster_rank:  %d
        #3 
        #12: cluster_size:  %d
        #1 
        #13: run_size:  %d
        #20 
        #14: rseed1:    %d
        #980868135 
        #15: rseed2:    %d
        #18826794 
        #16: rmsd:  %f
        #4.979233 
        #a couple of forms of energies: ASK LINDY what these are
        #17: binding_energy: %f
        #-16.653694 
        #18: docking_energy: %f
        #-18.776157 
        #19: translation: 3%f
        #5.005155 -2.431130 15.439371 
        #20: quaternion: 4%f
        #0.009171 0.617751 0.786320 -170.668379 
        #21: number of torsions followed by n values
        #12 -67.91 -107.72 101.99 -56.92 176.25 21.67 48.10 24.66 -3.65 12.54 -10.66 -3.68 
        #write the constant stuff
        fileptr.write("17 18 19 1/23/2001 7:27:34 AM  1/23/2001 7:27:34 AM 1.00 3.00 3.05")
        for item in ['run_rank', 'run', 'cluster_rank', 'cluster_size',
                    'run_size', 'rseed1', 'rseed2']:
            if not hasattr(self, item):
                fileptr.write(' 1')
            elif getattr(self, item):
                fileptr.write(" %d" %getattr(self, item))
            else:
                fileptr.write(" 10")

        #write the RMSD for this conformation
        fileptr.write(" %f" %(round(self.getRMSD(),6)))

        for item in [self.binding_energy, self.docking_energy, 
                    self.translation[0] + self.origin[0], 
                    self.translation[1] + self.origin[1], 
                    self.translation[2] + self.origin[2], 
                    self.quaternion[0], self.quaternion[1], 
                    self.quaternion[2], self.quaternion[3]]:
            fileptr.write(" %f" %item)
        fileptr.write(" %d" %len(self.torsions))
        for tor in self.torsions:
            fileptr.write(" %f" % tor)
        #at the end add a newline
        fileptr.write('\n')


#    def writeRes4(self, fileptr):
#        #eg
#        fileptr.write("17 18 19 1/23/2001 7:27:34 AM  1/23/2001 7:27:34 AM 1.00 3.00 3.05")
#        for item in ['run_rank', 'run', 'cluster_rank', 'cluster_size',
#                    'run_size', 'rseed1', 'rseed2']:
#            if not hasattr(self, item):
#                fileptr.write(' 1')
#            elif getattr(self, item):
#                fileptr.write(" %d" %getattr(self, item))
#            else:
#                fileptr.write(" 10")
#
#        #write the RMSD for this conformation
#        fileptr.write(" %f" %(round(self.getRMSD(),6)))
#
#        for item in [self.binding_energy, self.docking_energy, 
#                    self.translation[0] + self.origin[0], 
#                    self.translation[1] + self.origin[1], 
#                    self.translation[2] + self.origin[2], 
#                    self.quaternion[0], self.quaternion[1], 
#                    self.quaternion[2], self.quaternion[3]]:
#            fileptr.write(" %f" %item)
#        fileptr.write(" %d" %len(self.torsions))
#        for tor in self.torsions:
#            fileptr.write(" %f" % tor)
#        #at the end add a newline
#        fileptr.write('\n')


def dist(c1, c2):
    d = Numeric.array(c2) - Numeric.array(c1)
    ans = math.sqrt(Numeric.sum(d*d))
    return round(ans, 3)

def getAngle(at1, at2, at3 ):
    pt1 = Numeric.array(at1.coords, 'f')
    pt2 = Numeric.array(at2.coords, 'f')
    pt3 = Numeric.array(at3.coords, 'f')
    v1 = Numeric.array(pt1 - pt2)
    v2 = Numeric.array(pt3 - pt2)
    dist1 = math.sqrt(Numeric.sum(v1*v1))
    dist2 = math.sqrt(Numeric.sum(v2*v2))
    sca = Numeric.dot(v1, v2)/(dist1*dist2)
    if sca>1.0:
        sca = 1.0
    elif sca<-1.0:
        sca = -1.0
    ang =  math.acos(sca)*180./math.pi
    return round(ang, 5)

def build_dict(mol):
    d = {}
    bonds = build_bond_dict(mol)
    angles = build_angle_dict(mol)
    d['bonds'] = bonds
    d['angles'] = angles
    return d

def build_bond_dict(mol):
    bonds = {}
    for a in mol.allAtoms:
        for b in a.bonds:
            n1 = b.atom1.number
            n2 = b.atom2.number
            k = (n1,n2)
            if n2<n1:
                k = (n2,n1)
            bonds[k] = dist(b.atom1.coords, b.atom2.coords)
    return bonds


def build_angle_dict(mol):
    angles = {}
    for a1 in mol.allAtoms:
        #print "a1=", a1.name, ':', a1.number
        n1 = a1.number
        #print "n1=",n1
        for b in a1.bonds:
            a2 = b.neighborAtom(a1)
            #print "a2=", a2.name, ':', a2.number
            n2 = a2.number
            #print "n2=",n2
            for b2 in a2.bonds:
                a3 = b2.neighborAtom(a2)
                if a3==a1:
                    continue
                else:
                    n3 = a3.number
                    #print "a3=", a3.name, ':', a3.number
                    k = (a1,a2,a3)
                    kk = (a1.number, a2.number, a3.number)
                    if a1.number>a3.number:
                        kk = (a3.number, a2.number, a1.number)
                    if kk not in angles.keys():
                        new_a = getAngle(a1,a2,a3)
                        angles[kk] = new_a
    return angles
                

class ConformationHandler:
    """This class is bolted onto a Docking instance to manage the conformations which 
    result from an AutoDock experiment, one 'Conformation' per completed run.
    """
    def __init__(self, mol, origin, buildAngDict=0):
        """
        """
        self.mol = mol
        self.origin = origin
        self.conformations = []
        #conformations are 1-based to fit with autodock 1-based rank
        self.current_conf = None
        mol.allAtoms.addConformation(mol.allAtoms.coords)
        self.confIndex = len(mol.allAtoms[0]._coords) - 1
        if buildAngDict:
            self.angDict = build_angle_dict(mol)
        if hasattr(mol,'torTree'):
            mol.stoc = self.stoc = StateToCoords(mol, origin, self.confIndex)


    def test_conformation(self, conf, confInd, cutoff=0.003, angle_cutoff=15, verbose=False):
        found_different_bond_length = False
        found_different_angle = False
        self.set_conformation(conf)
        test_dict = build_dict(self.mol)
        #test_dict['angles'] = dict
        #test_dict['bonds'] = dict
        for entry in test_dict.keys():
            tD = test_dict[entry]
            rD = self.ref_dict[entry]
            for k,v in tD.items():
                refv = rD[k]
                mag_diff = abs(refv-v)
                if entry=='bonds':
                    if mag_diff>cutoff:
                        found_different_bond_length = True
                        if verbose: print confInd, ": Distance %d%s-%d%s differs: % 6.4f %6.4f \n" \
                            %(k[0]-1,self.mol.allAtoms[k[0]-1].name,k[1]-1, self.mol.allAtoms[k[1]-1].name, refv,v)
                else:
                    if mag_diff>angle_cutoff:
                        found_different_angle = True
                        if verbose: print confInd, ": Angle %d%s-%d%s-%d%s differs: % 6.4f %6.4f \n" \
                            %(k[0]-1,self.mol.allAtoms[k[0]-1].name,k[1]-1, self.mol.allAtoms[k[1]-1].name,k[2]-1, self.mol.allAtoms[k[2]-1].name, refv,v)
        valid = (not found_different_bond_length) and (not found_different_angle)
        if not found_different_bond_length and verbose:
            print confInd, ": no bonds found differing in length more than ", cutoff
        if not found_different_angle and verbose:
            print confInd, ": no angles found differing more than ", angle_cutoff
        return valid
            

    def validate(self, conformations, cutoff=0.003, angle_cutoff=5, verbose=False):
        if len(self.mol.allAtoms[0].bonds)==0:
            self.mol.buildBondsByDistance()
        if not hasattr(self, 'ref_dict'):
            self.ref_dict = build_dict(self.mol)
        valid_conformations = []
        bad_conformations = []
        for ix, conf in enumerate(conformations):
            valid = self.test_conformation(conf,ix,cutoff, angle_cutoff, verbose)
            if valid:
                valid_conformations.append(conf)
            else:
                bad_conformations.append(conf)
        return valid_conformations, bad_conformations
            

    def add(self, clist, keywords=ResultParser.keywords, validate=False, verbose=False, filename=None ):
        """Create/add conformations to the handler.

        clist is a list of dictionaries probably created by
        a subclass of AutoDockTools.ResultParser.
        keywords is the set of keys whose values should become
        Conformation attributes. ResultParser.keywords is the miminal
        and default set.
        """
        #some conformations may not have a translation:
        #in that case want to set translation to (0.,0.,0.)
        #if they do, want to remove it from translation
        #d.get('trn_x', self.origin[0]) returns either trn_x or self.origin[0]
        #thus when you subtract self.origin[0] from what you get you either get
        #corrected translation OR 0. which is what you want... i think
        for d in clist:
            translation = (d.get('trn_x', self.origin[0])- self.origin[0],
                           d.get('trn_y', self.origin[1])- self.origin[1],
                           d.get('trn_z', self.origin[2])- self.origin[2])

            quaternion = (d.get('qtn_nx',0),
                          d.get('qtn_ny',0),
                          d.get('qtn_nz',0),
                          d.get('qtn_ang_deg',0))
            
            newConformation = Conformation(mol = self.mol,
                origin = self.origin,
                # remove origin from AutoDocks reporting of translation
                translation =  translation,
                quaternion =  quaternion,
                torsions = d.get('torsion_values',[]),
                coords = d.get('coords',None))

            newConformation.quaternion0 = (d.get('quaternion_nx',0),
                          d.get('quaternion_ny',0),
                          d.get('quaternion_nz',0),
                          d.get('quaternion_nw',0))

            # add key to newConformation with value from d
            # d is the parser-returned dict
            # missing keys default to None courtesy of d.get
            for key in keywords:
                setattr(newConformation, key, d.get(key, None))
            #NEW: add 'ligand_efficiency' 
            binding_energy = newConformation.binding_energy
            newConformation.ligand_efficiency = 0
            newConformation.filename = filename
            if not hasattr(self.mol, 'lenNonHAtoms'):
                self.mol.lenNonHAtoms = len(self.mol.allAtoms.get(lambda x: x.element!='H'))
            if self.mol.lenNonHAtoms>0:
                newConformation.ligand_efficiency = round(binding_energy/self.mol.lenNonHAtoms, 2)
            # now append to our list
            self.conformations.append(newConformation)
            self.original_conformations = self.conformations

        if validate:
            if verbose: print "pre_validate:len(confs)=", len(self.conformations)
            self.conformations, self.badconformations = self.validate(self.conformations, verbose=verbose)
            if verbose: print "post_validate:len(confs)=", len(self.conformations)


    def set(self, index):
        self.set_conformation(self.conformations[index])


    def set_conformation(self, conf, index=None):
        """Tell the molecule about its new conformation

        """
        # drop the coords into the stoc.confIndex slot of mol._coords
        #self.mol.allAtoms.setConformation(self.confIndex)
        allAtoms = self.mol.allAtoms
        coords = conf.getCoords()
        if not index:
            index = self.confIndex
        allAtoms.updateCoords(coords[:], index)
        #z = self.confIndex
        #for n in range(len(coords)):
        #    allAtoms[n]._coords[z] = coords[n]

        #save possible per atom energy information
        if hasattr(conf, 'estat_energies') and conf.estat_energies and len(conf.estat_energies)==len(allAtoms):
            for n in range(len(allAtoms)):
                allAtoms[n].estat_energy = conf.estat_energies[n]

        if hasattr(conf,'vdw_energies') and conf.vdw_energies and len(conf.vdw_energies)==len(allAtoms):
            for n in range(len(allAtoms)):
                allAtoms[n].vdw_energy = conf.vdw_energies[n]

        if hasattr(conf,'total_energies') and conf.total_energies and len(conf.total_energies)==len(allAtoms):
            for n in range(len(allAtoms)):
                allAtoms[n].total_energy = conf.total_energies[n]

        if hasattr(conf, 'ad4_estat_energies') and conf.ad4_estat_energies and len(conf.ad4_estat_energies)==len(allAtoms):
            for n in range(len(allAtoms)):
                allAtoms[n].ad4_estat_energy = conf.ad4_estat_energies[n]

        if hasattr(conf,'ad4_vdw_energies') and conf.ad4_vdw_energies and len(conf.ad4_vdw_energies)==len(allAtoms):
            #these lump together vdw, hb and dsolv
            for n in range(len(allAtoms)):
                allAtoms[n].ad4_vdw_energy = conf.ad4_vdw_energies[n]

        if hasattr(conf,'ad4_energies') and conf.ad4_energies and len(conf.ad4_energies)==len(allAtoms):
            for n in range(len(allAtoms)):
                allAtoms[n].ad4_energy = conf.ad4_energies[n]

        #conformations are 1-based to fit with autodock 1-based rank
        index = self.conformations.index(conf)
        self.current_conf = conf
        #self.current_conf = index + 1
           

    def writeTrjFile(self, outfptr, run=0, cycle=0, temp=0, istep=0,
                        lastmove=0, eint = 0):
        #first write the header stuff:
        outfptr.write('ntorsions %d\n' % len(self.conformations[0].torsions))
        outfptr.write('run %d\n' % run)
        outfptr.write('cycle %d\n' % cycle)
        outfptr.write('temp %f\n' % temp)
        #what are istep, lastmove and eint????
        istep = 1
        lastmove = 'a'
        for c in self.conformations:
            c.writeTrj(self, outfptr, istep, lastmove)
            istep = istep + 1


    def writeResFile(self, outfptr, istep=0, lastmove=0, eint=0):
        # FIX THIS TO GET id info: istep, lastmove, eint etc
        for c in self.conformations:
            c.writeRes(outfptr, istep, lastmove, eint)
        


class PopulationHandler(ConformationHandler):
    """This class is a specialized ConformationHandler
    designed to handle many individuals in  populations created in a docking.

    """
    def __init__(self, mol, origin):
        """
        """
        ConformationHandler.__init__(self, mol, origin)
        self.individuals = self.conformations
        #individuals conformations are 1-based to fit with autodock 1-based rank
        #self.current_ind = self.conformations
        self.current_pop_ind = 0
        self.all_populations = []
        if len(self.conformations):
            self.all_populations.append(self.conformations)


    def set_current_pop(self, ind):
        assert ind<len(self.all_populations), 'index not in range of populations'
        self.current_pop_ind = ind
        self.conformations = self.all_populations[ind]
        self.individuals = self.conformations
    

    def add(self, clist, keywords=ResultParser.keywords):
        """Create/add conformations to the handler.

        clist is a list of dictionaries probably created by
        a subclass of AutoDockTools.ResultParser.
        keywords is the set of keys whose values should become
        Conformation attributes. ResultParser.keywords is the miminal
        and default set.
        """
        #some conformations may not have a translation:
        #in that case want to set translation to (0.,0.,0.)
        #if they do, want to remove it from translation
        #d.get('trn_x', self.origin[0]) returns either trn_x or self.origin[0]
        #thus when you subtract self.origin[0] from what you get you either get
        #corrected translation OR 0. which is what you want... i think
        confs = []
        for d in clist:
            translation = (d.get('trn_x', self.origin[0])- self.origin[0],
                           d.get('trn_y', self.origin[1])- self.origin[1],
                           d.get('trn_z', self.origin[2])- self.origin[2])

            quaternion = (d.get('qtn_nx',0),
                          d.get('qtn_ny',0),
                          d.get('qtn_nz',0),
                          d.get('qtn_ang_deg',0))

            newConformation = Conformation(mol = self.mol,
                origin = self.origin,
                # remove origin from AutoDocks reporting of translation
                translation =  translation,
                quaternion =  quaternion,
                torsions = d.get('torsion_values',[]),
                coords = d.get('coords',None))

            # add key to newConformation with value from d
            # d is the parser-returned dict
            # missing keys default to None courtesy of d.get
            for key in keywords:
                setattr(newConformation, key, d.get(key, None))

            # now append to our list
            confs.append(newConformation)
        self.all_populations.append(confs)
        self.conformations = self.all_populations[0]
        if len(self.all_populations)==1:
            self.current_pop_ind = 0

    

    

class State:
    """Storage class for the state of any molecule
    """
    def __init__(self, molecule = None, id = 0, origin = (0.,0.,0.),
                 translation = None, quaternion = None,
                 torsions = []):
        """Constructor for class describing molecule State

        self.id = Int              # identifier
        origin = Point()           # center for quaternion motion
        translation = Point()      # translation of ligand center
        quaternion = Quaternion() # orientation of ligand
        torsions = []              # list of torsions in degrees
        """
        self.mol = molecule
        self.id = id
        self.origin = origin
        self.translation = translation
        self.quaternion = quaternion
        self.torsions = torsions
        self.ntorsions = len(torsions)



class AutodockState(State):
    """AdState represents the state of a molecule.
        self.nstep = Int           # number of steps in trajectory cycle
        self.acc_rej_code = ''     # accept/reject code
        e_binding = Float          # intermolecular energy of ligand + macromolecule
                                   # PLUS torsional free energy
        e_total = Float            # energy of ligand + macromolecule
        e_internal = Float         # energy of ligand alone
        e_inter = Float            # Final Intermolecular Energy(from dlg)
        e_intra = Float            # Final Internal Energy of Ligand(from dlg)
        e_tors = Float             # Torsional Free Energy (from dlg)
       The class stores the pdb file name and state
       variables.
    """
    def __init__(self, molecule = None, id = 0, nstep = None, acc_rej_code = None,
                 e_binding = None, e_total = None, e_internal = None, 
                 origin = (0.,0.,0.), translation = None, 
                 quaternion = None, torsions = [], e_inter = None, 
                 e_intra = None, e_tors = None):
        """Constructor for the AutodockState class.
        """
        State.__init__(self, molecule = molecule, id = id, origin = origin,
                translation = translation, quaternion = quaternion, 
                torsions = torsions)
        self.nstep = nstep
        self.acc_rej_code = acc_rej_code
        self.e_binding = e_binding
        self.e_total = e_total
        self.e_internal = e_internal
        self.e_inter = e_inter
        self.e_intra = e_intra
        self.e_tors = e_tors
        assert hasattr(self.mol, 'torTree'), 'molecule has no torTree'



import os

class CADD_Vision:
    """
    This class supports creating Vision network using Opal web services to
    perform computational vistual screening experiments
    """

    def __init__(self):

        # create an instance of Vision without a GUI
        from Vision.VPE import NoGuiExec
        self.ed = NoGuiExec()

    def run(self):
        from NetworkEditor.net import Communicator
        self.net.communicator = Communicator(self.net)
        print 'Communicator listening on port:', self.net.communicator.port

        import socket
        #f = open(argv[0]+'.sock', 'w')
        #f.write("%s %i"%(socket.gethostbyname(socket.gethostname()),
        #                 self.net.communicator.port))
        #f.close()
        print "%s %i"%(socket.gethostbyname(socket.gethostname()),
                       self.net.communicator.port)
                       
        self.net.run()


class CADD_VistualScreening(CADD_Vision):
    """
    virtual screenign based on VS_net.py

    This network allows a user to specify:
      - a directory of structures
      - a set of ligands taken from a public ligand database
      - filtering options for the ligands
      - whether to preserve the charges or not

    the parameters are set using hte correspondign methods
        .setStructureDirectory(path)
        .setLigandDB(string)
        .setFilter(string)
        .setPreseveCharges(0/1)
    """

    def __init__(self):

        CADD_Vision.__init__(self)

        # name used by Vision for this network
        name = 'vs_opal'
        
        # find the path to CADD networks in the AutoDockTools package
        from mglutil.util.packageFilePath import findFilePath
        netpath = findFilePath('CADD', 'AutoDockTools')

        # build full path to VS_net.py network
        filename = os.path.join(netpath, 'VisionNetworks', 'VS_net.py')

        # load the network
        self.net = self.ed.loadNetworkNOGUI(name, filename)


    def setStructureDirectory(self, path):
        assert os.path.exists(path)
        assert os.path.isdir(path)

        # get handle to input ports
        nodes = self.net.getNodeByName('GetStructuresFromDir')
        assert len(nodes)==1
        node = nodes[0]

        # a get a handle to the port
        port = node.getInputPortByName('directory')

        # set the widget to the specified value without triggering execution
        port.widget.set(path, run=0)

        
    def setLigandDB(self, name):
        assert name in [
            'sample', 'sample2', 'NCI_DS1', 'NCI_DS2', 'NCIDS_SC', 'oldNCI',
            'chembridge_building_blocks', 'drugbank_neutraceutics',
            'drugbank_smallmol', 'fda_approved', 'human_metabolome',
            'asinex', 'otava', 'zinc_natural_products.prop']

        # get handle to input ports
        nodes = self.net.getNodeByName('PublicServerLigandDB')
        assert len(nodes)==1
        node = nodes[0]

        # a get a handle to the port
        port = node.getInputPortByName('server_lib')

        # set the widget to the specified value without triggering execution
        port.widget.set(name, run=0)


    def setFilter(self, filterMode):
        assert filterMode in ['None', 'default', 'Lipinski-like',
                              'Drug-like', 'Drug-like frag', 'Cistom']
        # get handle to input ports
        nodes = self.net.getNodeByName('FilterLigandsNode')
        assert len(nodes)==1
        node = nodes[0]

        # a get a handle to the port
        port = node.getInputPortByName('filterMode')

        # set the widget to the specified value without triggering execution
        port.widget.set(filterMode, run=0)


    def setPreseveCharges(self, onOff):
        assert onOff in (0,1)
        
        # get handle to input ports
        nodes = self.net.getNodeByName('PreserveCharges?')
        assert len(nodes)==1
        node = nodes[0]

        # a get a handle to the port
        port = node.getInputPortByName('button')

        # set the widget to the specified value without triggering execution
        port.widget.set(onOff, run=0)







#  $Id: test_AD41analyze.py,v 1.9 2010/08/24 21:48:49 sargis Exp $




import unittest, glob, os, types, Pmv.Grid
import time
from string import find,split
mv = None
ct = 0
totalCt = 53
###totalCt = 65
klass = None

class AD41analyze_BaseTests(unittest.TestCase):
    """
    setUp + tearDown form a fixture: working environment for the testing code
    """
    
        
    def startViewer(self):
        global mv
        #print 'in test_AD startViewer'
        if mv is None:
            from MolKit import Read
            import Tkinter
            from Pmv.moleculeViewer import MoleculeViewer
            mv = MoleculeViewer(trapExceptions = False,
                                withShell=0
                                )
            #mv.loadModule('autotors41Commands', 'AutoDockTools')
            #mv.loadModule('autogpf41Commands', 'AutoDockTools')
            #mv.loadModule('autodpf41Commands', 'AutoDockTools')
            #8/18:
            #mv.loadModule('autoanalyze41Commands', 'AutoDockTools')
            mv.loadModule('deleteCommands', 'Pmv')
            mv.loadModule('selectionCommands', 'Pmv')
            #change warningMsg format
            mv.setUserPreference(('warningMsgFormat', 'printed'), log = 0)
            from AutoDockTools.autotorsCommands import AdtSetMode
            mv.addCommand(AdtSetMode(), 'ADTSetMode')
            mv.ADTSetMode("AD4.0")
            mv.ADTSetMode("AD4.2")
        self.mv = mv


    def setUp(self):
        """
        clean-up
        """
        global klass, mv
        name = self.__class__.__name__
        if not klass:
            klass = name
        elif klass != name:
            print 'SETUP: destroying mv'
            if mv:
                # reload AutoDockTools and  Pmv modules:
                from AutoDockTools import autotors4Commands, autoflex4Commands,\
                      autotors41Commands, autoflex41Commands
                reload(autotors4Commands)
                reload(autoflex4Commands)
                reload(autotors41Commands)
                reload(autoflex41Commands)
                
                import Pmv
                allmod = dir(Pmv)
                for m in allmod:
                    if m.endswith("Commands"):
                        exec("if type(Pmv.%s) == types.ModuleType: reload(Pmv.%s)" % (m,m))
                mv.Exit(0)
                if hasattr(self, 'mv'):
                    del self.mv
                mv = None
            klass = name
        if not hasattr(self, 'mv'):
            self.startViewer()


    def tearDown(self):
        """
        clean-up
        """
        #print 'in AD41analyze tearDown'
        global ct, totalCt
        #reset docked
        from AutoDockTools.Docking import Docking
        for key in self.mv.dockings.keys():
            try:
                self.mv.AD41analyze_deleteDLG(key)
            except:
                pass
        self.mv.dockings = {}
        self.mv.docked = None
        #delete any molecules left due to errors
        for m in self.mv.Mols:
            try:
                self.mv.deleteMol(m)
            except:
                pass
        for g in self.mv.grids.values():
            #for clean up:
            #hide the box
            g.surfGUI.boxvVar.set(0)
            g.surfGUI.changeBox()
            #hide the grid
            if hasattr(g, 'srf') and g.srf:
                g.srf.Set(visible=0)
                g.srf.protected = False
            if g.geom:
                g.geom.protected = False
            if hasattr(g, 'box') and g.box:
                g.box.protected = False
            if hasattr(g, 'slices') and g.slices:
                for slice_list in g.slices.values():
                    for slice in slice_list: 
                        slice.protected = False
                delattr(g, 'slices')
                    
            self.mv.GUI.VIEWER.Redraw()
        #clean-out the dictionary
        self.mv.grids = {}
        ct = ct + 1
        #print 'ct =', ct
        if ct==totalCt:
            print 'destroying mv'
            self.mv.Exit(0)
            del self.mv



class AD41analyze_readDLGTests(AD41analyze_BaseTests):
    """
    setUp + tearDown form a fixture: working environment for the testing code
    """
       
    

   
    def test_readDlg(self):
        """reads a dlg and checks 
        """
        self.mv.AD41analyze_readDLG("ind.dlg")
        self.assertEqual(self.mv.docked.dlo_list[0].filename, "ind.dlg")
        self.assertEqual(self.mv.docked.ligMol.name, "ind_out")
  
   
    def test_confor(self):
        """checks for # of conformations
        """
        self.mv.AD41analyze_readDLG("ind.dlg")
        self.assertEqual(len(self.mv.docked.ch.conformations), 10)
  

    def test_delete(self):
        """checks everything is gone
        """
        self.mv.AD41analyze_readDLG("ind.dlg")
        self.mv.AD41analyze_deleteDLG('ind.dlg')
        self.assertEqual(self.mv.docked, None)
 

    def test_cluster(self):
        """checks has clusters
        """
        self.mv.AD41analyze_readDLG("ind.dlg")
        cd = self.mv.docked.clusterer.clustering_dict
        self.assertEqual(cd.keys()[0], 0.5)
        self.assertEqual(len(cd[0.5][0]), 4)
        dlo = self.mv.docked.dlo_list[0]
        self.assertEqual(dlo.hasClusters, 1)


    def test_macrofile(self):
        """checks for macrofile & macroStem
        """
        self.mv.AD41analyze_readDLG("ind.dlg")
        dlo = self.mv.docked.dlo_list[0]
        self.assertEqual(dlo.macroFile, 'hsg1.pdbqs')
        self.assertEqual(dlo.macroStem, 'hsg1')
   
    def test_emptyDlg(self):
       """tests empty dlg entry
       """
       self.assertRaises(IOError,self.mv.AD41analyze_readDLG," ")
 
    def test_readMissingDlg(self):
       """checks non-existent dlg
       """
       self.assertRaises(IOError,self.mv.AD41analyze_readDLG,"fish.dlg")
  
    def test_delete_with_no_docking(self):   
       """checks when one dlg entered and requested for deletion of other
       """
       self.assertRaises(KeyError, self.mv.AD41analyze_deleteDLG,"flh.dlg")


    def test_delete_with_docking(self):   
        """ deleting non-existing name
        """
        self.mv.AD41analyze_readDLG("ind.dlg")
        self.assertRaises(KeyError, self.mv.AD41analyze_deleteDLG,"flh.dlg")


    def test_selectdockeddlg(self):
        """tests not to add to previous
        """
        self.mv.AD41analyze_readDLG("ind.dlg")
        self.mv.AD41analyze_readDLG("flash.dlg",0)
        self.assertEqual(len(self.mv.docked.dlo_list),1)
        self.mv.docked = None
        self.mv.dockings = {}
        #delete any molecules left due to errors
        for m in self.mv.Mols:
            self.mv.deleteMol(m)
        
        
    def test_selectdlg(self):
        """checkligmolnem selected current docked one
        """
        self.mv.AD41analyze_readDLG("ind.dlg")
        self.mv.AD41analyze_readDLG("flash.dlg",0)
        self.mv.AD41analyze_selectDLG("ind.dlg")
        self.assertEqual(self.mv.docked.ligMol.name, "ind_out")
        self.mv.docked = None
        self.mv.dockings = {}
        #delete any molecules left due to errors
        for m in self.mv.Mols:
            self.mv.deleteMol(m)
    

    def test_selectnon_existentdlg(self):
        """check for non-existent selection
        """
        self.mv.AD41analyze_readDLG("ind.dlg")
        self.assertRaises(KeyError,self.mv.AD41analyze_selectDLG,"soap.dlg")
   

    def test_selectemptydlg(self):
        """"check for empty selection
        """
        self.assertRaises(KeyError,self.mv.AD41analyze_selectDLG," ")


    def test_selectdirdlg(self):
        """"check reading all dockinglogs in a directory into separate dockings
        """
        self.mv.AD41analyze_readAllDLGInDirectory("./",0, ask=0 )
        dlgfiles = glob.glob("./*.dlg")
        self.assertEqual(len(self.mv.dockings), len(dlgfiles))


    def test_selectdirdlgseparate(self):
        """"check reading all dockinglogs in a directory into 1 docking
        """
        self.mv.AD41analyze_readAllDLGInDirectory("./",1, ask=0 )
        self.assertEqual(len(self.mv.dockings),1)
     

    def test_showmolecule(self):
        """adds a macromolecule to the viewer and checks length of dir
        """
        self.mv.AD41analyze_readDLG("ind.dlg")
        # self.mv.Mols[0]="ind.dlg"
        #self.mv.Mols[1]="hsg1.pdbqs"
        print len(self.mv.Mols)
        for m in self.mv.Mols:
            print m.name
        self.assertEqual(hasattr(self.mv.docked,'macroMol'), False)        
        self.mv.AD41analyze_readMacromolecule("hsg1.pdbqs")
        self.assertEqual(hasattr(self.mv.docked,'macroMol'), True)        
        self.assertEqual(len(self.mv.Mols),2)
        self.assertEqual(self.mv.docked.macroMol, self.mv.Mols[-1])        
        #delete any molecules left due to errors
        for m in self.mv.Mols:
            self.mv.deleteMol(m)
    

    def test_choose_macromolecule(self):
        """tests choose macro molecule
        """
        self.mv.AD41analyze_readDLG("ind.dlg")
        self.mv.AD41analyze_readMacromolecule("hsg1.pdbqs")
        self.mv.AD41analyze_chooseMacromolecule("ind_out")
        self.assertEqual(self.mv.docked.macroMol.name,"ind_out")
        self.mv.AD41analyze_chooseMacromolecule("hsg1")
        self.assertEqual(self.mv.docked.macroMol.name,"hsg1")
              
    
    def test_nomacromolecule(self):
        """checks for non-existent macromolecule
        """
        self.mv.AD41analyze_readDLG("ind.dlg")
        self.mv.AD41analyze_readMacromolecule("hsg1.pdbq")
        self.assertEqual(hasattr(self.mv.docked,'macroMol'), False)        
        self.assertEqual(len(self.mv.Mols),  1)        
        #delete any molecules left due to errors
        for m in self.mv.Mols:
            self.mv.deleteMol(m)
    

    def test_existingmacromolecule(self):
        """checks length when existing mol called
        """
        self.mv.AD41analyze_readDLG("ind.dlg")
        le=len(self.mv.Mols)
        self.mv.AD41analyze_readMacromolecule("hsg1.pdbqs")
        self.assertEqual(len(self.mv.Mols),le+1)
        self.mv.AD41analyze_readMacromolecule("hsg1.pdbqs")
        self.assertEqual(len(self.mv.Mols),le+1)
        #delete any molecules left due to errors
        for m in self.mv.Mols:
            self.mv.deleteMol(m)


    def test_dockings(self):
        """tests for current docking instance
        """
        filename = "ind.dlg"
        self.mv.AD41analyze_readDLG(filename)
        self.mv.AD41analyze_readMacromolecule("hsg1.pdbqs")
        self.assertEqual(os.path.basename(self.mv.docked.dlo_list[0].filename), filename)

    
    def test_dockingspheres(self):
        """checks ShowDockingAs Spheres: that number of vertices==number of conformations
        """
        self.mv.AD41analyze_readDLG("ind.dlg")
        c = self.mv.AD41analyze_showDockingsAsSpheres
        c(["ind.dlg"]) #calls GUI command which opens its widget
        c.lb.selection_set(0) #set the widget's listbox selection to 1st entry
        c.updateVis()   #updates the dockedSpheres geometry for ligmol
        ligand = self.mv.docked.ligMol
        #get a handle to the geometry
        spheresGeom = ligand.geomContainer.geoms['dockedSpheres']
        #check that the geometry has the correct number of spheres
        self.assertEqual(len(spheresGeom.vertexSet),len(mv.docked.ch.conformations))
        #close the widget
        c.form.withdraw()
    

    def test_emtydlgsdockingspheres(self):
        """checks emptys dlgs docking spheres
        """
        self.mv.AD41analyze_readDLG("ind.dlg")
        self.assertRaises(KeyError,self.mv.AD41analyze_showDockingsAsSpheres,[" "])


    def test_dockingspheresnosset(self):
        """ check that dockedSpheres are not visible if docking is not selected in listbox
        """
        self.mv.AD41analyze_readDLG("ind.dlg")
        c = self.mv.AD41analyze_showDockingsAsSpheres
        c(["ind.dlg"]) #calls GUI command which opens its widget
        c.updateVis()   #updates the dockedSpheres geometry for ligmol
        ligand = self.mv.docked.ligMol
        #get a handle to the geometry
        spheresGeom = ligand.geomContainer.geoms['dockedSpheres']
        self.assertEqual(spheresGeom.visible, False)
        #close the widget
        c.form.withdraw()
    

    def test_dockingspheres_flexres(self):
        """ check that dockedSpheres are centered on ligand atoms only
        """
        self.mv.AD41analyze_readDLG("tut_ind.dlg")
        c = self.mv.AD41analyze_showDockingsAsSpheres
        c(["tut_ind.dlg"]) #calls GUI command which opens its widget
        c.lb.selection_set(0) #set the widget's listbox selection to 1st entry
        c.updateVis()   #updates the dockedSpheres geometry for ligmol
        ligand = self.mv.docked.ligMol
        #get a handle to the geometry
        spheresGeom = ligand.geomContainer.geoms['dockedSpheres']
        verts = spheresGeom.vertexSet.vertices.array[:]
        #check that first two vertices are expected values based on lig atoms only
        self.assertAlmostEqual(verts[0][0], 2.61951017)
        self.assertAlmostEqual(verts[0][1], 6.15904093)
        self.assertAlmostEqual(verts[0][2],-7.75912237)
        self.assertAlmostEqual(verts[1][0], 2.55975509)
        self.assertAlmostEqual(verts[1][1], 6.20373487)
        self.assertAlmostEqual(verts[1][2],-7.4846735)
        #close the widget
        c.form.withdraw()


###    def evalmol(self):
###        """test calling command mode of autodock"""
###        
###        self.mv.AD41analyze_readDLG("ind.dlg")
###        c=self.mv.AD41analyze_epdbMolecule
###        estFreeEnergy = self.mv.AD41analyze_epdbMolecule('hsg1_ind.dpf',['d1.pdbq'])
###        try:
###            self.assertEqual(round(estFreeEnergy,1), -14.4)
###        except:
###            #currently there is a bug in epdb in autodock3 compiled for sgi machines
###            self.assertEqual(round(estFreeEnergy,1), 4.4)
         

    def test_chooseDockedConformations(self):
        """ test choosing lowest energy conformation"""
        self.mv.AD41analyze_readDLG("ind.dlg")
        c=self.mv.AD41analyze_chooseDockedConformations
        c.guiCallback() #opens the listchooser widget
        c.chooser.lb.select_set(1) #want to show lowest energy
        c.chooser.show_cb()  #this changes conformation of ligand
        lowestEnergyConf = self.mv.docked.clusterer.clustering_dict[0.5][0][0]
        #check that the current conformation is the lowest energy one
        self.assertEqual(lowestEnergyConf, self.mv.docked.ch.current_conf)
        c.chooser.done_cb()  #this closes widget, i hope


###    def test_getChart(self):
###        """test that all the names of the ligands and macromolecules are in the chart
###"""
###        dlgs = ['ind.dlg','flash.dlg','flap.dlg','tut_ind.dlg']
###        for fn in dlgs:
###            self.mv.AD41analyze_readDLG(fn)
###        #self.mv.AD41analyze_readAllDLGInDirectory('./',0)
###        c = self.mv.AD41analyze_getChart
###        c(dlgs)
###        #c(['ind.dlg','flash.dlg','flap.dlg','tut_ind.dlg'])
###        #the buttons have names of ligands and macromolecules
###        namesOfButtons = c.ifd.entryByName.keys()
###        for d in self.mv.dockings.values():
###            self.assertEqual(d.ligMol.name in namesOfButtons, True)
###            self.assertEqual(d.dlo_list[0].macroStem in namesOfButtons, True)
###        c.close()
   

    def test_grids(self):
        """reads a map and checks
        """
        self.mv.AD41analyze_readDLG("ind.dlg")
        c=self.mv.AD41analyze_showGridIsocontours
        c("hsg1.C.map")
        m=mv.grids.values()[0]
        self.assertEqual(m.__class__,Pmv.Grid.AutoGrid)
        c.Close_cb()
        #cleanup: remove the command's ifd so it has to rebuild
        delattr(c, 'ifd')
       
   
    def test_gridsvset(self):
        """checks length of vertexset
        """
        self.mv.AD41analyze_readDLG("ind.dlg")
        c=self.mv.AD41analyze_showGridIsocontours
        c("hsg1.C.map")
        m=mv.grids.values()[0]
        self.assertEqual(len(m.srf.vertexSet), 2263)
        #before UTisocontour removed duplicate vertices
        #self.assertEqual(len(m.srf.vertexSet),8114)
        c.Close_cb()
       

    def test_gridsvisible(self):
        """checks visiblity of grid
        """
        self.mv.AD41analyze_readDLG("ind.dlg")
        c=self.mv.AD41analyze_showGridIsocontours
        c("hsg1.C.map")
        m=mv.grids.values()[0]
        self.assertEqual(m.srf.visible,1)
        c.Close_cb()
        #cleanup: remove the command's ifd so it has to rebuild
        delattr(c, 'ifd')
    

    def test_gridnonexistent(self):
       """checks with empty map name
       """
       self.mv.AD41analyze_readDLG("ind.dlg")
       self.assertRaises(IOError,self.mv.AD41analyze_showGridIsocontours," ", ask=0)
              
       
    def test_gridswrongname(self):
       """checks with wrong map name
       """
       self.mv.AD41analyze_readDLG("ind.dlg")
       self.assertRaises(IOError,self.mv.AD41analyze_showGridIsocontours,"hiu.dlg ",ask=0)


    def test_gridtwiceread(self):
        """checks when read twice map name
        """
        self.mv.AD41analyze_readDLG("ind.dlg")
        c=self.mv.AD41analyze_showGridIsocontours
        returnValue = c("hsg1.C.map")
        self.assertEqual(returnValue.__class__, Pmv.Grid.AutoGrid)
        returnValue.surfGUI.boxvVar.set(0)
        returnValue.surfGUI.changeBox()
        returnValue.srf.Set(visible=0)
        returnValue2 = c("hsg1.C.map")
        self.assertEqual(returnValue2, 'ERROR')
        c.Close_cb()
        self.mv.AD41analyze_deleteDLG("ind.dlg")
        #cleanup: remove the command's ifd so it has to rebuild
        delattr(c, 'ifd')
   

    def test_read_extra_grid(self):
        """ check reading in a grid not used in docking for comparison purposes."""
        self.mv.AD41analyze_readDLG("ind.dlg")
        c = self.mv.AD41analyze_showGridIsocontours
        c("extra.S.map")
        m = mv.grids.values()[0]
        self.assertEqual(m.__class__,Pmv.Grid.AutoGrid)
        c.Close_cb()
        #cleanup: remove the command's ifd so it has to rebuild
        delattr(c, 'ifd')
        
    def test_conformations_showstates_spwattr(self):
       """ checks for spw attr 
       """
       self.mv.AD41analyze_readDLG("ind.dlg")
       self.mv.AD41analyze_showStates("ind_out")
       self.assertEqual(hasattr(self.mv.Mols[0],"spw"),True)
       self.mv.Mols[0].spw.Close_cb()
        
    def test_conformations_showstates_toplevel_winfo(self):
       """ checks toplevel wininfo is mapped
       """
       self.mv.AD41analyze_readDLG("ind.dlg")
       self.mv.AD41analyze_showStates("ind_out")
       self.mv.Mols[0].spw.applyState(0)
       self.assertEqual(self.mv.Mols[0].spw.form.root.winfo_ismapped(),1)
       self.mv.Mols[0].spw.Close_cb()
        
    def test_conformations_showstates_sequencelist(self):
        """checks sequencelist equals to # of conformations
        """
        self.mv.AD41analyze_readDLG("ind.dlg")
        self.mv.AD41analyze_showStates("ind_out")
        self.assertEqual(len(self.mv.Mols[0].spw.sequenceList),len(self.mv.docked.ch.conformations))
        self.mv.Mols[0].spw.Close_cb()

    def test_conformations_showstates_sequencelist_sameorder(self):
        """checks conformations are in same order in both sequence list and
        docked ch list
        """
        self.mv.AD41analyze_readDLG("ind.dlg")
        self.mv.AD41analyze_showStates("ind_out")
        for i in xrange(len(self.mv.Mols[0].spw.sequenceList)):
            self.assertEqual(self.mv.Mols[0].spw.sequenceList[i],self.mv.docked.ch.conformations[i])
        self.mv.Mols[0].spw.Close_cb()

    def test_conformations_showstates_applystates(self):
        """checks coords changing the apply states
        """
        self.mv.AD41analyze_readDLG("ind.dlg")
        self.mv.AD41analyze_showStates("ind_out")
        atom1=self.mv.Mols[0].allAtoms[0]
        self.assertEqual(round(atom1.coords[0],3),4.080)  
        self.assertEqual(round(atom1.coords[1],3),6.701)
        self.assertEqual(round(atom1.coords[2],3),-3.117)
        self.mv.Mols[0].spw.applyState(-1)
        self.assertEqual(round(atom1.coords[0],3),-3.884)  
        self.assertEqual(round(atom1.coords[1],3),-2.421)
        self.assertEqual(round(atom1.coords[2],3),-3.900)
        self.mv.Mols[0].spw.Close_cb()


    def test_Showstates_total_energy(self):
        """ checks total energy
        """
        self.mv.AD41analyze_readDLG("ind.dlg")
        self.mv.AD41analyze_showStates("ind_out")
        self.mv.Mols[0].spw.applyState(0)
        atom1=self.mv.Mols[0].allAtoms[0]
        self.assertEqual(round(atom1.total_energy,3),-0.240)
        self.mv.Mols[0].spw.Close_cb()


    def test_showstates_binding_energy(self): 
        """checks binding energy
        """
        self.mv.AD41analyze_readDLG("ind.dlg")
        self.mv.AD41analyze_showStates("ind_out")
        self.assertEqual(self.mv.docked.ch.conformations[0].binding_energy,-14.34)
        self.mv.Mols[0].spw.Close_cb()
    
    def test_showstates_docking_energy(self): 
        """"checks docking energy
        """
        self.mv.AD41analyze_readDLG("ind.dlg")
        self.mv.AD41analyze_showStates("ind_out")
         
        self.assertEqual(self.mv.docked.ch.conformations[0].docking_energy,-18.84)
        self.mv.Mols[0].spw.Close_cb()


###class AD41analyze_readDiversityResultFiles(AD41analyze_BaseTests):


###    def setUp(self):
###        """
###        clean-up
###        """
###        if not hasattr(self, 'mv'):
###            self.startViewer()
###        self.mv.AD41analyze_readStates('diversity1483_x1hpv.dpf','2176.res')
###        self.mv.AD41analyze_showStates("diversity1483")



###    def test_read_conformations_from_file_atmNum(self):
###        """ checks # of atoms in dpf  after reading
###        using readStates
###  
###        """
###        self.assertEqual(len(self.mv.Mols[-1].atmNum),21)    
###        self.mv.Mols[-1].spw.Close_cb()
###  

###    def test_read_conformations_from_file_length(self):
###        """checks length in sequence list after reading
###        using readStates
###        """
###        self.assertEqual(len(self.mv.Mols[-1].spw.sequenceList), 20)
###        self.mv.Mols[-1].spw.Close_cb()
###  

###    def test_read_conformations_from_file_coords(self):
###        """checks coords are different after applystate()    
###        """
###        m=self.mv.Mols[-1].ROOT    
###        #self.assertEqual(round(m._coords[-1][0],3),0.002)
###        #self.assertEqual(round(m._coords[-1][1],3),-0.004)
###        #self.assertEqual(round(m._coords[-1][2],3),0.002)
###        #self.mv.Mols[-1].spw.applyState(0) 
###        self.assertEqual(round(m._coords[-1][0],3),-1.688)
###        self.assertEqual(round(m._coords[-1][1],3),0.611)
###        self.assertEqual(round(m._coords[-1][2],3),3.759)
###        self.mv.Mols[-1].spw.applyState(3) 
###        self.assertEqual(round(m._coords[-1][0],3),-1.649)
###        self.assertEqual(round(m._coords[-1][1],3), 0.684)
###        self.assertEqual(round(m._coords[-1][2],3),3.813)
###        self.mv.Mols[-1].spw.Close_cb()


###    def test_read_conformations_from_file_binding_energy(self):
###        """checks docking energy after reading from read States 
###        """        
###        self.assertEqual(round(self.mv.docked.ch.conformations[0].binding_energy,3), -8.944)
###        self.mv.Mols[-1].spw.Close_cb()
###       

###    def test_read_conformations_from_file_docking_energy(self):
###        """checks docking energy after reading from readStates       
###        """
###        self.assertEqual(round(self.mv.docked.ch.conformations[0].docking_energy, 3),-9.513)
###        self.mv.Mols[-1].spw.Close_cb()
   


###class AD41analyze_read_write_result_files(AD41analyze_BaseTests):
###        
###    def setUp(self):
###        """
###        clean-up
###        """
###        if not hasattr(self, 'mv'):
###            self.startViewer()
###        self.mv.AD41analyze_readDLG("ind.dlg", ask=0)
###        if not os.path.exists('test_ind.res'):
###            self.mv.AD41analyze_writeStates("ind_out",'test_ind.res')
###        self.mv.AD41analyze_readStates('hsg1_ind.dpf', 'test_ind.res', 0, ask=0)



###    def tearDown(self):
###        """
###        clean-up
###        """
###        print 'in read_write tearDown'
###        global ct, totalCt
###        #reset docked
###        from AutoDockTools.Docking import Docking
###        self.mv.docked = None
###        for key in self.mv.dockings.keys():
###            self.mv.AD41analyze_deleteDLG(key)
###        self.mv.dockings = {}
###        #delete any molecules left due to errors
###        #for m in self.mv.Mols:
###        #    self.mv.deleteMol(m)
###        ct = ct + 1
###        #print 'ct =', ct
###        if ct==totalCt:
###            print 'destroying mv'
###            self.mv.Exit(0)
###            del self.mv


###    def test_write_result_file_exists(self):
###        """6:checks file exists after writing  result file
###        """
###        self.assertEqual(os.path.exists('test_ind.res'),True)
###        cmd = "rm test_ind.res"
###        os.system(cmd)

###         
###    def test_write_dockings_result_file(self):
###        """2:checks dockings after writing  result file
###        """
###        d1 = self.mv.dockings['ind.dlg']
###        d2 = self.mv.dockings['test_ind.res']
###        self.assertEqual(d1.__class__, d2.__class__)
###    

###    def test_write_docking_result_file_dlolist_length(self):
###        """1:checks length of dlo_lists are equal after writing result file
###        """
###        d1 = self.mv.dockings['ind.dlg']
###        d2 = self.mv.dockings['test_ind.res']
###        self.assertEqual(len(d1.dlo_list), len(d2.dlo_list))


###    def test_write_result_file_dockings_docking_energy(self):
###        """4:checks clustering dict of d1 and d2 conformations docking energies
###        are equal after writing  result file
###        """
###        d1 = self.mv.dockings['ind.dlg']
###        d2 = self.mv.dockings['test_ind.res']
###        d1_conf_list = d1.clusterer.clustering_dict[0.5][0]
###        d2_conf_list = d2.ch.conformations
###        self.assertEqual( d1_conf_list[0].docking_energy, d2_conf_list[0].docking_energy)
###        self.assertEqual(d1_conf_list[1].docking_energy,d2_conf_list[1].docking_energy)
###   

###    def test_write_result_file_dockings_number_of_conformations(self):
###        """5:checks lengths of conformations after writing  result file
###        """
###        d1 = self.mv.dockings['ind.dlg']
###        d2 = self.mv.dockings['test_ind.res']
###        self.assertEqual(len(d1.ch.conformations), len(d2.ch.conformations))
###      

###    def test_write_result_file_compare_ligMols(self):
###        """3:checks names of ligmols after writing  result file
###        """
###        d1 = self.mv.dockings['ind.dlg']
###        d2 = self.mv.dockings['test_ind.res']
###        self.assertEqual(len(d1.ligMol.allAtoms), len(d2.ligMol.allAtoms))
###        self.assertEqual(d1.ligMol.name+'_1', d2.ligMol.name)
       


class AD41analyze_clusterings(AD41analyze_BaseTests):
        
    def setUp(self):
        """
        clean-up
        """
##         if not hasattr(self, 'mv'):
##             self.startViewer()
        apply(AD41analyze_BaseTests.setUp, (self, ), {})
        self.mv.AD41analyze_readDLG("ind.dlg")
        self.mv.AD41analyze_showClusteringStates("ind_out")
           

    def test_show_clusterings_length(self):
        """checks length clustering_dict
        """
        d = self.mv.docked
        v = d.clusterer.clustering_dict[0.5]
        self.assertEqual(len(v),2)
        self.assertEqual(len(v[0]),4)
        self.assertEqual(len(v[1]),6)


    def test_show_clusterings_lowest_energy(self):
        """checks lowest energy of clustering dict and conformations are same
        """
        d = self.mv.docked
        v = d.clusterer.clustering_dict[0.5]
        self.assertEqual(round(v[0][0].docking_energy,3),
                      round(d.ch.conformations[5].docking_energy,3))
        

    def test_show_clusterings_binding_energy(self):
        """checks cluterings binding energies
        """
        d = self.mv.docked
        v = d.clusterer.clustering_dict[0.5]
        self.assertEqual(round((v[0][0].binding_energy),3),-14.280)
        
        
    def test_show_clusterings_docking_energy(self):
        """checks clusterings docking energies
        """
        d = self.mv.docked
        v = d.clusterer.clustering_dict[0.5]
        self.assertEqual(round((v[0][0].docking_energy),3),-18.900)
    
        
    def test_show_clusterings_shows_histogram(self):
        """checks widgetfor showing histograms exists
        """
        d = self.mv.docked
        m = d.ligMol
        #self.mv.AD41analyze_showClusteringStates("ind_out-2")
        self.assertEqual(m.clustNB.master.winfo_exists(), 1)
        #self.assertEqual(m.clustNB.master.winfo_ismapped(), 1)
        m.clustNB.exit()
        self.assertEqual(m.clustNB.master.winfo_exists(), 0)
    

    def test_make_clusterings_at_high_rmsd(self):
        """checks clusterings at high rmsd
        """
        d = self.mv.docked
        self.mv.AD41analyze_selectDLG('ind.dlg')
        self.mv.AD41analyze_makeClustering([12.0], 'docking', None)
        v = d.clusterer.clustering_dict[12.]
        self.assertEqual(len(v), 1)
        self.assertEqual(len(v[0]), 10)


    def test_make_clusterings_at_low_rmsd(self):
        """checks clusterings at low rmsd
        """
        
        d = self.mv.docked
        self.mv.AD41analyze_selectDLG('ind.dlg')
        self.mv.AD41analyze_makeClustering([0.1], 'docking', None)
        v = d.clusterer.clustering_dict[0.1]
        self.assertEqual(len(v), 10)
        self.assertEqual(len(v[0]), 1)

    def test_write_clusterings_file_length_of_all_lines(self):
        """checks no . of ind_out all lines and conformations are same
        """
        
        d = self.mv.docked
        fptr=open("test_ind.clust")
        alllines=fptr.readlines()
        self.assertEqual(len(alllines),len(d.ch.conformations)+1)
        
    def test_write_clusterings_file_length_of_split_lines(self):
        """checks first line of ind_out is same length as clusterer.dict
        """
        
        d = self.mv.docked
        fptr=open("test_ind.clust")
        alllines=fptr.readlines()
        from string import split
        ll=split(alllines[0])
        v = d.clusterer.clustering_dict
        self.assertEqual(len(ll),len(v.keys())+5)
     
    def test_write_clusterings_file_energy_used(self):
        """checks both are using docking energies
        """
        
        d = self.mv.docked
        c1=d.clusterer
        fptr=open("test_ind.clust")
        alllines=fptr.readlines()
        from string import split
        ll=split(alllines[0])
        ll[-1]=c1.energy_used
        self.assertEqual(ll[-1],c1.energy_used)
    


class AD41analyze_check_coordinates(AD41analyze_BaseTests):

    def test_current_coords_of_atom0_bug(self):
        """checks ats[0]._coords[0] = ats[0].coords
        """
        self.mv.AD41analyze_readDLG("ind.dlg")
        ats0 = self.mv.allAtoms[0]
        old_ats0_coords =ats0._coords
        old_ats0coords  =ats0.coords
        self.mv.AD41analyze_showStates("ind_out")
        self.mv.Mols[0].spw.applyState(1)
        new_ats0_coords =ats0._coords
        new_ats0coords  =ats0.coords
        self.assertEqual(old_ats0coords,old_ats0_coords[0])
        #It should be new_ats0coords = new_ats0_coorsd[0],but we are getting
        self.assertEqual(new_ats0coords!=new_ats0_coords[0],True)
        self.mv.Mols[0].spw.Close_cb()
        
    def test_write_current_coords_bug(self):
        """checks when 1-1 docked conformation is choosed ,coords of atom0 =
        atom0 's coords in currentcoords file 
        """
        self.mv.AD41analyze_readDLG("ind.dlg")
        ats0 = self.mv.allAtoms[0]
        c = self.mv.AD41analyze_chooseDockedConformations
        c.guiCallback() #opens the listchooser widget
        c.chooser.lb.selection_clear(0,'end')
        c.chooser.lb.select_set(1) #want to show lowest energy
        c.chooser.show_cb()
        old_ats0coords  =ats0.coords
        self.mv.writePDB("ind_out","curcoords.pdb")
        fptr  =open("curcoords.pdb")
        alllines = fptr.readlines()
        #for i in range(0,len(alllines)):
	        #if find(str(alllines),'HETATM   1') >= 0:
                #pos = i
		        #break;
        split(str(alllines[0]))
        #these should be equal
        self.assertEqual(ats0.coords[0] != alllines[0][6],True)
        self.assertEqual(ats0.coords[1] != alllines[0][7],True)
        self.assertEqual(ats0.coords[2] != alllines[0][8],True)
        c.chooser.done_cb()
        os.system('rm -f curcoords.pdb')
        


class AD41analyze_showBindingSite(AD41analyze_BaseTests):
    """
     check AD41analyze_showBindingSite for expected display
    """
   
    def test_showBindingSite(self):
        """
        check for expected number of atoms in various displayed geometries

        """
        self.mv.AD41analyze_readDLG("ind.dlg")
        docking = self.mv.docked
        self.mv.AD41analyze_readMacromolecule("hsg1.pdbqs")
        self.mv.AD41analyze_showStates("ind_out") 
        command = self.mv.AD41analyze_showBindingSite
        
        #import pdb; pdb.set_trace()
        
        command('ind.dlg') 
        self.mv.GUI.VIEWER.Redraw()
        # input conformation of ligand clashes with receptor
        lig = self.mv.Mols[0]
        lig_gcg = lig.geomContainer.geoms
        lig_gca = lig.geomContainer.atoms
        macro = self.mv.Mols[1]
        macro_gcg = macro.geomContainer.geoms
        macro_gca = macro.geomContainer.atoms
        #12/13/07:changed macro close res display from lines to s&b
        self.assertEqual(len(macro_gca['sticks']), 118)
        #self.assertEqual(len(macro_gca['bonded']), 118)
        self.assertEqual(len(macro_gca['ResidueLabels']), 13)
        #check for secondary structure
        #turn on showSS:
        command.showSS.set(1)
        command('ind.dlg')
        self.assertEqual(len(lig_gca['bonded']), 49)
        self.assertEqual(len(lig_gca['ind_out_msms']), 49)
        self.assertEqual(len(lig_gca['cpk']), 0)
        #check for expected number of hbonds in input conf.
        self.assertEqual(len(lig.allAtoms.get(lambda x: hasattr(x, 'hbonds'))),7)
        #change to docked conformation
        lig.spw.applyState(2)
        #check for secondary structure
        self.assertEqual(len(macro_gca['ResidueLabels']), 14)
        #check for expected number of hbonds
        self.assertEqual(len(lig.allAtoms.get(lambda x: hasattr(x, 'hbonds'))), 6)
        # undisplay Msms in order to check for expected number of close-contact (8) 
        # and hbonding atoms(3) in the ligand for this conformation
        var = command.showMsms
        var.set(0)
        lig.spw.applyState(1)
        #command.update(docking, ('msmsatoms', var, self.mv.displayMSMS))
        self.mv.GUI.VIEWER.Redraw()
        # spheres on close contact displayed, spheres on hbonds not displayed
        self.assertEqual(len(lig_gca['cpk']), 23)
        # turn on 'display spheres on hbonding atoms' command.showHbatSpheres
        command.showHbatSpheres.set(1)
        lig.spw.applyState(1)
        #command.update(docking, ('hb_macro_ats', command.showHbatSpheres, self.mv.displayCPK))
        self.mv.GUI.VIEWER.Redraw()
        self.assertEqual(len(lig_gca['cpk']), 26)
        # turn off 'display spheres on close contact atoms' command.showCloseContactSpheres
        command.showCloseContactSpheres.set(0)
        lig.spw.applyState(1)
        #command.update(docking, ('non_hbas', command.showCloseContactSpheres, self.mv.displayCPK))
        self.mv.GUI.VIEWER.Redraw()
        # hbonding atoms(3) in the ligand for this conformation
        self.assertEqual(len(lig_gca['cpk']), 3)
        #redisplay Msms
        var.set(1)
        lig.spw.applyState(1)
        #command.update(docking, ('msmsatoms', var, self.mv.displayMSMS))
        self.mv.GUI.VIEWER.Redraw()
        # when msms surface for ligand displayed, no spheres are drawn on
        # ligand hbonding and close contact atoms
        self.assertEqual(len(lig_gca['cpk']), 0)



if __name__ == '__main__':
    unittest.main()
    #test_cases = [
    #        AD41analyze_BaseTests,
    #        AD41analyze_readDLGTests,
### #        AD41analyze_readDiversityResultFiles, DEPRECATED
    #        AD41analyze_read_write_result_files,
    #        AD41analyze_clusterings,
    #        AD41analyze_check_coordinates,
    #        AD41analyze_showBindingSite,
    #        ]
    #unittest.main( argv=([__name__ ,] + test_cases) )










#        

#   








#!/usr/bin/env python
#
# 
#
# $Header: /opt/cvs/python/packages/share1.5/AutoDockTools/Utilities24/prepare_covalent_flexres.py,v 1.9 2014/02/05 17:18:00 rhuey Exp $
#
import os 
from PyBabel.atomTypes import AtomHybridization
from mglutil.math.rigidFit import RigidfitBodyAligner
from MolKit import Read
from MolKit.molecule import AtomSet
from MolKit.bondSelector import RotatableBondSelector, AmideBondSelector
from MolKit.bondSelector import GuanidiniumBondSelector, LeafBondSelector
from mglutil.math.rotax import rotax
from math import sqrt

from AutoDockTools.MoleculePreparation import AD4FlexibleReceptorPreparation, AD4LigandPreparation

try:
    import Numeric
except:
    import numpy.oldnumeric as Numeric

def dist(a1, a2):
    c1 = Numeric.array(a1.coords)
    c2 = Numeric.array(a2.coords)
    d = c2-c1
    return sqrt(sum(d*d))


def getBondedAtoms(atom):
    l = AtomSet()
    for b in atom.bonds:
        l.append(b.neighborAtom(atom))
    return l


def detect_clashes(atomset1, atomset2, cutoff=1.2, verbose=False):  #check this v. autodock/mdist.h
    #exclude bonded pairs
    clashes = []
    for at1 in atomset1:
        if verbose: print "processing ", at1.name,
        bonded = getBondedAtoms(at1)
        if verbose: print "len(bonded)=", len(bonded),
        for at2 in atomset2:
            if verbose: print "vs ", at2.name
            if at2 in bonded:
                continue
            d = dist(at1,at2)
            BondOrderSum = at1.bondOrderRadius + at2.bondOrderRadius
            if verbose: print 'distance = % 6.4f vs BondOrderSum*cutoff = % 6.4f' %(d, BondOrderSum*cutoff)
            if d <= BondOrderSum * cutoff:
                if verbose: print "%s - %s : % 6.4f - % 6.4f" %(at1.name, at2.name, d, BondOrderSum)
                clashes.append(((at1, at2),d))
    return clashes


def setAutoFlexFields(res):
    #process residues
    if hasattr(res, 'setup'): 
        return
    res.setup = 1
    res.atoms.used = 0
    res.atoms.bonds[0].possibleTors = 0
    res.atoms.bonds[0].activeTors = 0
    backbone_names = ['C','N','O','HN','HN1','HN2', 'HA', 
                'H1','H2','H3','HO', 'H']
    #includes CA
    sidechain = res.atoms.get(lambda x: x.name not in backbone_names)
    res.sideChain = sidechain
    bondlist = res.bondlist = sidechain.bonds[0]
    bondlist.leaf = 0
    bondlist.possibleTors = 0
    bondlist.activeTors = 0
    rbs = RotatableBondSelector()
    rotatables = rbs.select(bondlist)
    for b in rotatables:
        b.possibleTors = 1
        b.activeTors = 1
    amides = AmideBondSelector().select(bondlist)
    for b in amides:
        b.activeTors = 0
        b.possibleTors = 1
    guanidiniums = GuanidiniumBondSelector().select(bondlist)
    for b in guanidiniums:
        b.activeTors = 0
        b.possibleTors = 1
    leaves = LeafBondSelector().select(bondlist)
    for b in leaves:
        b.activeTors = 0
        b.possibleTors = 0
    res.torscount = len(bondlist.get(lambda x: x.activeTors==1))
    #this field is not used in AutoDock4
    res.torsdof = res.torscount
    res.torscount = len(bondlist.get(lambda x: x.activeTors==1))
    res.torsdof = res.torscount
    
    caAtoms = res.atoms.get(lambda x: x.name=='CA')
    #get returns an AtomSet
    if caAtoms:  #this checks for len(caAtoms)
        res.rootlist = caAtoms
    else:
        res.rootlist = AtomSet([res.atoms.get(lambda x: x._uniqIndex == 0)[0]])
        res.sideChain = res.atoms


def setRelativeTorsion(atom1, atom2, angle): #mov_atoms=None):
    #rat1, rat2, ??30degrees??
    mol = atom1.top
    if mov_atoms is None:
        mov_atoms = mol.subTree(atom1, atom2, mol.allAtoms)
    assert len(mov_atoms)
    mov_coords = Numeric.array(mov_atoms.coords)
    lenCoords = len(mov_coords)
    x = Numeric.array(atom1.coords)
    y = Numeric.array(atom2.coords)
    rot = (angle * 3.14159/180.)%(2 * Numeric.pi)
    matrix = rotax(x, y, rot)
    _ones = Numeric.ones(lenCoords, 'f')
    _ones.shape = (lenCoords,1)
    mov_coords = Numeric.concatenate((mov_coords, _ones),1)
    newcoords = Numeric.dot(mov_coords, matrix)
    nc = newcoords[:,:3].astype('f')
    for i in range(lenCoords):
        at = mov_atoms[i]
        at._coords[at.conformation] = nc[i].tolist()



if __name__ == '__main__':
    import sys
    import getopt


    def usage():
        "Print helpful, accurate usage statement to stdout."
        print "Usage: prepare_covalent_flexres.py -r receptor_filename -l ligand_to_superimpose_filename -b atom1_atom2_atom3 -R residue_name"
        print "    Description of command..."
        print "         -r   receptor_filename"
        print "         -l   ligand_filename"
        print "         -b   ligand atom1_atom2_atom3 to superimpose (eg SD_CG_CB)"
        print "         -R   name of receptor residue onto which the bond atoms "
        print "                   in the ligand will be superimposed"
        print "    Optional parameters:"
        print "        [-B]  alternate atom1_atom2_atom3 in receptor residue to superimpose "
        print "            (default uses atom1_atom2_atom3 names specified with -b flag)"
        print "        [-v]  verbose output"
        print "        [-g pdbqt_filename] (rigid output filename)"
        print "        [-x pdbqt_filename] (flexible output filename)"
        print "        [-n pdbqt_filename] (flexible output filename)"
        print "                                              "
        print "  NOTE the order of the atoms specified for the bond is crucial:"   
        print "        '-b atom1_atom2_atom3' "
        print "   specifies this superposition:"
        print "       LIGAND-atom1-atom2-atom3"   
        print "              atom1-atom2-atom3-RECEPTOR"   
        print "  Process removes atom1-atom2-atom3-RECEPTOR bond and adds bond LIGAND-atom1-atom2-atom3-RECEPTOR"
        print "                                   ^                                                    ^"

    # process command arguments
    try:
        opt_list, args = getopt.getopt(sys.argv[1:], 'r:l:b:R:B:g:x:n:vh')
    except getopt.GetoptError, msg:
        print 'prepare_covalent_flexres.py: %s' %msg
        usage()
        sys.exit(2)

    # initialize required parameters
    #-r: receptor
    receptor_filename =  None
    #-l: ligand
    ligand_filename =  None
    #-b: atom1_atom2_atom3
    atom1_atom2_atom3 =  None
    #-R: res_in_receptor
    res_in_receptor =  None
    # optional parameters
    ratom1_ratom2_ratom3 = None
    # optional parameters
    verbose = None
    #-g  : rigid output filename
    rigid_filename = None
    #-x  : flexible output filename
    flexres_filename = None 
    #-n :  newflexible output filename
    new_flexres_filename = None
    #'r:l:b:R:B:g:x:n:vh'
    for o, a in opt_list:
        #print "o=", o, " a=", a
        if o in ('-r', '--r'):
            receptor_filename = a
            #CHECK THAT IT IS ALREADY IN PDBQT FORMAT
            ext = os.path.splitext(receptor_filename)[1]
            if ext!=".pdbqt":
                error = receptor_filename + " not in .pdbqt format"
                raise error
            if verbose: print 'set receptor_filename to ', a
        if o in ('-l', '--l'):
            ligand_filename = a
            ext = os.path.splitext(ligand_filename)[1]
            if ext!=".pdbqt":
                error = ligand_filename + " not in .pdbqt format"
                raise error
            if verbose: print 'set ligand_filename to ', a
        if o in ('-b', '--b'):
            atom1_atom2_atom3 = a
            if verbose: print 'set atom1_atom2_atom3 to ', a
        if o in ('-R', '--R'):
            res_in_receptor = a
            if verbose: print 'set res_in_receptor to ', a
        if o in ('-B', '--B'):
            ratom1_ratom2_ratom3 = a
            if verbose: print 'set ratom1_ratom2_ratom3 to ', a
        if o in ('-v', '--v'):
            verbose = True
            if verbose: print 'set verbose to ', True
        if o in ('-g', '--g'):
            rigid_filename = a
            if verbose: print 'set rigid_filename to ', a
        if o in ('-x', '--x'):
            flexres_filename = a
            if verbose: print 'set flexres_filename to ', a
        if o in ('-n', '--n'):
            new_flexres_filename = a
            if verbose: print 'set new_flexres_filename to ', a
        if o in ('-h', '--h'):
            usage()
            sys.exit()


    if not  receptor_filename:
        print 'prepare_covalent_flexres: receptor filename must be specified!\n'
        usage()
        sys.exit()

    if not  ligand_filename:
        print 'prepare_covalent_flexres: ligand_filename to move must be specified!\n'
        usage()
        sys.exit()

    if not  atom1_atom2_atom3:
        print 'prepare_covalent_flexres: atom1_atom2_atom3 to superimpose must be specified!\n'
        usage()
        sys.exit()

    if not  res_in_receptor:
        print 'prepare_covalent_flexres: res_in_receptor onto which the ligand will superimposed must be specified!\n'
        usage()
        sys.exit()

    if not ratom1_ratom2_ratom3:
        ratom1_ratom2_ratom3 = atom1_atom2_atom3

    #initialize the receptor
    extension = os.path.splitext(receptor_filename)[1]
    if extension!=".pdbqt":
        print 'prepare_covalent_flexres: receptor file must be in .pdbqt format\n'
        usage()
        sys.exit()
    rM = Read(receptor_filename)[0]
    rM.buildBondsByDistance()
    if verbose: print 'read ', receptor_filename

    #initialize the flexible-residue-to-be in the receptor
    resPSet = rM.chains.residues.get(res_in_receptor)
    if not len(resPSet):
        msg= resP.name + " is not a residue in " + receptor_filename
        raise msg
    if len(resPSet)>1:
        msg= resP.name + " specifies more than one residue:" + resP.full_name()
        raise msg
    resP = resPSet[0]
    if verbose: print "receptor residue is %s which has %d atoms" %(resP.full_name(), len(resP.atoms))

    #initialize the ligand
    lM = Read(ligand_filename)[0]
    if verbose: print 'read ', ligand_filename
    lM.buildBondsByDistance()            # try to assign bond order;set to 1 if exception
    #lM.allAtoms.bonds[0].bondOrder = 1   #@@@@@@@@CHECK THIS!!!!@@@@@@@@@
    ah = AtomHybridization()
    ah.assignHybridization(lM.allAtoms)

    #process the ligand:
    #(1)locate specified atom1, atom2 and atom3  SD-CG-CB
    #NB: it is assumed that atom3 has exactly 2 bonds
    at_names = atom1_atom2_atom3.split('_')
    assert len(at_names)==3
    at1Set = lM.allAtoms.get(at_names[0])
    if not len(at1Set):
        msg= at_names[0] + " is not in " + ligand_filename
        raise msg
    at1 = at1Set[0] #SD, closest to ligand, farthest from receptor
    if verbose: print "first ligand atom to superimpose is %s" %(at1.full_name())
    at2Set = lM.allAtoms.get(at_names[1])
    if not len(at2Set):
        msg =  at_names[1] + " is not in " + ligand_filename
        raise msg
    at2 = at2Set[0] #CG
    if verbose: print "second ligand atom to superimpose is %s" %(at2.full_name())
    at3Set = lM.allAtoms.get(at_names[2])
    if not len(at3Set):
        msg =  at_names[2] + " is not in " + ligand_filename
        raise msg
    at3 = at3Set[0] #CB, farthest from ligand, closest to receptor
    if verbose: print "third ligand atom to superimpose is %s" %(at3.full_name())

    #(2)setup shortcut to these two atoms !IN ORDER!
    lig_atoms = AtomSet([at1,at2,at3])
        
    #process the receptor: 
    #(1) locate specified receptor residue
    resSet = rM.chains.residues.get(res_in_receptor)
    if not len(resSet):
        msg = res_in_receptor + " is not in " + receptor_filename
        raise msg
    if len(resSet)!=1:
        msg = res_in_receptor + " selected more than one residue:" + resSet.name
        raise msg
    # SD-CG-CB
    #(2) locate specified ratom1 in that residue
    rat_names = ratom1_ratom2_ratom3.split('_')
    assert len(rat_names)==3
    rat1Set = resP.atoms.get(rat_names[0])
    if not len(rat1Set):
        msg= rat_names[0] + " is not in " + resP.full_name()
        raise msg
    rat1 = rat1Set[0] #SD
    if verbose: print "first receptor atom for superposition is  %s" %(rat1.full_name())
    #(3) locate specified ratom2 in that residue
    rat2Set = resP.atoms.get(rat_names[1])
    if not len(rat2Set):
        msg =  rat_names[1] + " is not in " + resP.full_name()
        raise msg
    rat2 = rat2Set[0] #CG
    if verbose: print "second receptor atom for superposition is  %s" %(rat2.full_name())
    #(4) locate specified ratom3 in that residue
    rat3Set = resP.atoms.get(rat_names[2])
    if not len(rat3Set):
        msg =  rat_names[2] + " is not in " + resP.full_name()
        raise msg
    rat3 = rat3Set[0] #CB, closest to receptor
    if verbose: print "third receptor atom for superposition is  %s" %(rat3.full_name())
    #(5)setup shortcut to these two atoms !IN ORDER!
    ref_atoms = AtomSet([rat1,rat2,rat3])
    if verbose:
        print "for superposition: compute matrix to fit: "
        print "lig_atoms =[at1 = %s, at2 = %s, at3 = %s]" %(at1.full_name(), at2.full_name(), at3.full_name())
        print "onto "
        print "ref_atoms =[rat1 = %s, rat2 = %s, rat3 = %s]" %(rat1.full_name(), rat2.full_name(), rat3.full_name())
    if verbose: print "all input is valid!"
    #(6)locate 'bonds to break' which is the bond from rat3 not to rat2 , here CB: -SD--CG--CB==OtherAtom
    bond_to_break = None
    for b in rat3.bonds:
        otherAt = b.neighborAtom(rat3)
        if otherAt!=rat2:
            bond_to_break = b
            atom_to_attach = otherAt
            if verbose: print "found bond_to_break in receptor => %s" %(b.atom1.name +':'+ b.atom2.name)
            # break the bond
            if verbose: print "receptor atom for ligand attachment in residue %s is %s" %(resP.full_name(),otherAt.full_name())
            otherAt.bonds.remove(b)
            rat3.bonds.remove(b)
            if verbose: print "removed %s-%s bond from %s and from %s"%(b.atom1.name, b.atom2.name, otherAt.full_name(), rat3.full_name())
    if bond_to_break is None:
        print "unable to locate the bond to break in receptor residue %s: expected a bond from %s to an atom other than %s"%(resP.full_name(), rat3.name, rat2.name)
        raise 'invalid input'

    #setup for the superposition
    ref_coords = ref_atoms.coords 
    lig_coords = lig_atoms.coords
    #(1) create the tool
    rigidfitAligner = RigidfitBodyAligner()
    rigidfitAligner.setRefCoords(ref_coords)
    rigidfitAligner.rigidFit(lig_coords)
    #(2) get new coordinates for the moving atoms
    #    here all the ligand atoms move
    new_coords = rigidfitAligner.transformCoords(lM.allAtoms.coords)
    # get the correct shape for the coords (natoms,3)
    #new_coords.shape
    #(25, 4)
    new_coords=new_coords[:,:3] 
    #(3) add the new to ligand
    lM.allAtoms.addConformation(new_coords)
    confNum = len(lM.allAtoms[0]._coords)-1
    #(4) set the coordinates to use to the new ones 
    lM.allAtoms.setConformation(confNum)
    if verbose: print "new coordinates added to ligand"

    #add the ligand atoms other than at1 + at2 to the residue
    #(1) get the atoms to add
    atoms_to_adopt = lM.allAtoms[:]
    #atoms_to_adopt = lM.allAtoms.get(lambda x: not x in lig_atoms)
    if verbose: print "there are %d atoms_to_adopt" %(len(atoms_to_adopt))
    #(3) remove the bonds to the three excluded atoms
    ###attachment_atom.bonds.remove(bond_to_break)
    ###print "removed %s-%s bond from %s"%(bond_to_break.atom1.name, bond_to_break.atom2.name, attachment_atom.full_name())
    ###bond_to_break.neighborAtom(attachment_atom).bonds.remove(bond_to_break)
    if verbose: print 'before removing rat1,rat2 and rat3, residue %s has %d atoms.' %( resP.name, len(resP.atoms))
    #remove rat1, rat2 and rat3
    resP.atoms.remove(rat1)
    if verbose: print 'removed rat1 %s'%rat1.full_name()
    resP.atoms.remove(rat2)
    if verbose: print 'removed rat2 %s'%rat2.full_name()
    resP.atoms.remove(rat3)
    if verbose: print 'removed rat3 %s'%rat3.full_name()
    rM.allAtoms = rM.chains.residues.atoms
    if verbose: print 'after removing rat1,rat2 and rat3 but before adding ligand atoms, residue %s has %d atoms.' %( resP.name, len(resP.atoms))
    if verbose: print 'and the receptor has %d atoms'%len(rM.allAtoms)
    #keep handle to existing atoms for possible clashes
    res_atoms_to_check = resP.atoms[:]
    #(4) add the ligand atoms to the residue
    #for h in atoms_to_adopt:
    for h in lM.allAtoms:
        resP.adopt(h)
        if verbose: print "%s added to %s"%(h.full_name(), resP.full_name())
    if verbose: print 'after adding %d ligand atoms, residue %s has %d atoms.' %( len(lM.allAtoms), resP.name, len(resP.atoms))
    rM.allAtoms = rM.chains.residues.atoms
    if verbose: print 'and the receptor has %d atoms'%len(rM.allAtoms)
    #(5) add the bonds between at3 'CB' in the ligand and the attachment atom in the receptor 
    rM.addBond(at3, atom_to_attach)
    if verbose: 
        print 'built bond between %s from ligand and  atom %s  in residue %s' %( at3.name,atom_to_attach.name, resP.name)
        print "now the bonds of atom %s are:"%(at3.name)
        for b in at3.bonds:
            print " %s -%s"%(b.atom1.full_name(), b.atom2.full_name())

    #(6) do some necessary updates of the protein data structure
    rM.allAtoms = rM.chains.residues.atoms
    num_ats = len(rM.allAtoms)
    rM.allAtoms.number = range(1, num_ats+1)
    # LOOK FOR CLOSE CONTACTS BETWEEN atoms_to_adopt and res_atoms_to_check
    #if find close contacts, iterate rotation around bond
    #in steps of 30degrees using setAngle stuff...
    clashes = detect_clashes(res_atoms_to_check, atoms_to_adopt, verbose=verbose)
    print "@@@ detected %d clashes! @@@"%len(clashes)
    for cl in clashes:
        print "%s - %s = %6.4f"%(cl[0][0].name, cl[0][1].name, cl[1])
    if len(clashes)>0:
        print 'detected %d clash(s)'%(len(clashes))

    # Setup to use AutoDockTools code to write formatted flexres file
    #(1) detect rotatable bonds etc 
    setAutoFlexFields(resP)
    if verbose: print "set autodock fields in atoms in %s" %resP.name
    #to write output
    #(2) setup default output filename if necessary
    #-g  : rigid output filename
    if rigid_filename==None:
        rigid_filename = rM.name + '_rigid.pdbqt'
    if verbose: print "rigid filename is %s" %rigid_filename
    #-x  : flexible output filename
    if flexres_filename==None:
        flexres_filename = rM.name + '_flex.pdbqt'
    if verbose: print "flexible residue filename is %s" %flexres_filename
    #(3) use this class in AutoDockTools for writing formatted outputfiles... 
    frp = AD4FlexibleReceptorPreparation(rM, mode='interactive', 
                rigid_filename=rigid_filename,
                residues=[resP], flexres_filename=flexres_filename)

    frp.write_flex([resP], flexres_filename)
    frp.write_rigid(rM, rigid_filename)
    if new_flexres_filename is None:
        new_flexres_filename = "new_" + flexres_filename
    flexresMol = Read(flexres_filename)[0]
    LPO = AD4LigandPreparation(flexresMol, root=0, outputfilename = new_flexres_filename)

# To execute this command type:
# prepare_covalent_flexres.py -l filename -r receptor_filename -l ligand_to_superimpose_filename -b atom1_atom2 -R residue
#specific example:
#python -i prepare_covalent_flexres.py -r 1pwc-protein.pdbqt -l 1pwc-ligand.pdbqt -b OG_CB -R SER62  -g 1pwc_rigid.pdbqt -x 1pwc_flex.pdbqt





